/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "connectmediasearchsettings.h"
#include "config.h"

#include <mediaplugin.h>
#include <mediamodel.h>
#include <medium.h>
#include <searchplugin.h>

#include <QDir>
#include <QPluginLoader>

ConnectMediaSearchSettings::ConnectMediaSearchSettings(SearchPlugin* searchPlugin, QWidget* parent, Qt::WindowFlags flags)
	: SettingsWidget(parent, flags)
{
	this->searchPlugin = searchPlugin;
	
	setName(tr("Connect Media Attributes"));
	setupUi(this);
	connMed = qobject_cast<ConnectMediaSearch*>(connectMediaSearch);
	connectByPass(connMed);

	// create list with plugins
	QSettings settings;
  QString pluginDir = settings.value ( "pluginDir" ).toString();

  if ( pluginDir.isEmpty() ) {
    pluginDir = QString ( PLUGINDIR );
  }
  
  //load and test plugin
  QDir dir ( pluginDir );

  if ( !dir.exists() ) {
    emit error ( tr ( "Plugin dir does not exist!\n%1" ).arg ( pluginDir ) );
    return;
  }

  QStringList files = dir.entryList ( QDir::Files );

  if ( files.size() < 1 ) {
    emit error ( tr ( "No files found at \n%1" ).arg ( pluginDir ) );
    return;
  }

	QStringList types;
  foreach ( QString file, files ) {
		QPluginLoader loader ( dir.filePath(file) );
		QObject *plugin = loader.instance();
    if ( !plugin ){
			continue;
		}

		// first try to find media plugins
		MediaPlugin *newPlugin = qobject_cast<MediaPlugin *> ( plugin );

		if ( !newPlugin ){
			continue;
		}
		
		mediaPlugins.append(newPlugin);
		types << newPlugin->supportedMaterialTypes();
  }
  
  types.sort();
	typeEdit->addItems(types);
	
	connect(typeEdit, SIGNAL(currentIndexChanged(int)), this, SLOT(changeType()));
	changeType();
}

ConnectMediaSearchSettings::~ConnectMediaSearchSettings()
{

}

bool ConnectMediaSearchSettings::saveSettings()
{
	if(!connMed)
		return false;
	
	return connMed->saveSettings();
}

void ConnectMediaSearchSettings::changeType()
{
	if(!searchPlugin)
		return;
	
	connMed->saveSettings();
	
	QString type = typeEdit->currentText();
	foreach(MediaPlugin *plugin, mediaPlugins){
		if(!plugin->supportedMaterialTypes().contains(type))
			continue;
		
		MediaModel *model = plugin->newModel(type);
		Medium *medium = model->newMedium();
		delete model;
		connMed->setPlugins(medium, searchPlugin);
	}
	
}

#include "connectmediasearchsettings.moc"
