/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "etableview.h"
#include "etableview.moc"

#include "delegateforeignkey.h"

#include <QSettings>

ETableView::ETableView(QWidget* parent)
	: QTableView(parent)
{
	QSettings qSettings;
	setAlternatingRowColors(qSettings.value("alternatingRowColors").toBool());
}

void ETableView::setModel(QAbstractItemModel* model)
{
	deleteDelegates();
  QTableView::setModel(model);
}

void ETableView::reloadDelegates()
{
	QAbstractItemModel *mod = model();
	if(!mod)
		return;
	for(int i = 0; i < mod->columnCount(); i++){
		DelegateForeignKey *delegate = qobject_cast<DelegateForeignKey*>(itemDelegateForColumn(i));
		if(delegate){
			delegate->reloadForeignKeys();
		}
	}
}

void ETableView::deleteDelegates()
{
	// before we remove the model, we remove all delegates
	// and since delegates should not be shared,
	// we can delete them
	QAbstractItemModel *mod = model();
	if(!mod)
		return;
	
	for(int i = 0; i < mod->columnCount(); i++){
		QAbstractItemDelegate *delegate = itemDelegateForColumn(i);
		if(delegate){
			setItemDelegateForColumn(i, 0);
			delete delegate;
		}
	}
}

