/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef LOANMODEL_H
#define LOANMODEL_H

#include "libmacro.h"
#include <QAbstractItemModel>

class LoanItem;

#include <QModelIndex>
#include <QStringList>

class QAbstractItemView;
class QTableView;

/**
	* Manages loaning stuff
	*/
class EXPORT_DLL LoanModel : public QAbstractItemModel
{
	Q_OBJECT
	public:
		LoanModel (unsigned int materialId, unsigned int ownUserId, QObject *parent = 0 );
		LoanModel ( unsigned int ownUserId, QObject *parent = 0);
    ~LoanModel();

    // own API
		int indexOf(QString columnName) const;
    LoanItem *loanItemFromIndex ( QModelIndex index ) const;
// 		QModelIndex indexFromLoanItem(LoanItem *item);
// 		bool removeMedium(Medium *medium);
// 		bool removeMedium(QModelIndex index);
    bool store();
		bool reload();
		/**
			* removes all items from the model
			*/
		bool clear();
    void addItemDelegates(QAbstractItemView *view);
		void hideAndResizeColumns(QTableView *view);

    // QT API
    QVariant data ( const QModelIndex& index, int role = Qt::DisplayRole ) const;
    int columnCount ( const QModelIndex& parent = QModelIndex() ) const;
    int rowCount ( const QModelIndex& parent = QModelIndex() ) const;
    QModelIndex parent ( const QModelIndex& child ) const;
    QModelIndex index ( int row, int column, const QModelIndex& parent = QModelIndex() ) const;
    bool setData ( const QModelIndex& index, const QVariant& value, int role = Qt::EditRole );
    QVariant headerData ( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;
    Qt::ItemFlags flags ( const QModelIndex& index ) const;
    void sort ( int column, Qt::SortOrder order = Qt::AscendingOrder );
		
	protected slots:
		void errorByPass(QString msg);
		void warningByPass(QString msg);
		void informationByPass(QString msg);
		void itemChanged();
		
	private:
		void init(unsigned int materialId, unsigned int ownUserId, bool loadWithoutMaterial = false);
		void load();
		void setupHeader();
		
		QList<LoanItem*> items;
		unsigned int materialId, myUserId;
		QStringList headerList;

	signals:
		void error(QString msg);
    void warning(QString msg);
    void information(QString msg);
};

#endif // LOANMODEL_H
