/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DDIALOG_H
#define DDIALOG_H

#include "libmacro.h"

class MPushButton;

#include <QDialog>
#include <QMenu>
#include <QAction>
#include <QToolBar>
#include <QDockWidget>

class EXPORT_DLL MDialog : public QDialog
{
    Q_OBJECT

  public:
    MDialog ( QWidget *parent = 0, Qt::WindowFlags flags = 0 );
    ~MDialog();
    void setWindowTitle ( const QString &title );
		void setObjectName ( const QString &name );

  public slots:
    virtual void settingsChanged(){};
    virtual void print(){};
    virtual void save(){};
    virtual void doExport(){};
    virtual void redo(){};
    virtual void undo(){};
    virtual void cut(){};
    virtual void copy(){};
    virtual void paste(){};
    virtual void reload(){};
    virtual void reloadSettingsByPass();
		/**
			* Can be called by any subclass, to connect signal showHelp(QString) to helpButton.
			* It also set's the right icon, text and shortcut to the button
			*/
		void setHelp(QString page, MPushButton *helpButton = 0);

  protected:
		QString helpPage();
		void closeEvent(QCloseEvent *event);
    void connectByPass ( QObject *object );
    bool iconsAndText, alternatingRows;

  protected slots:
		void help();
		
    virtual void errorByPass ( const QString &string );
    virtual void warningByPass ( const QString &string );
    virtual void informationByPass ( const QString &string );

    virtual void addMenuByPass ( QMenu *newMenu, const QString &parent );
    virtual void delMenuByPass ( QMenu *del );
    virtual void addActionByPass ( QAction *newAction, const QString &parent );
    virtual void delActionByPass ( QAction *del );
    virtual void addToolbarByPass ( QToolBar *newToolbar );
    virtual void delToolbarByPass ( QToolBar *delToolbar );
    virtual void addNewDockWidgetByPass ( QDockWidget *widget, const Qt::DockWidgetArea &area );
    virtual void removeDockWidgetByPass ( QDockWidget *dockWidget );
    virtual void setCentralWidgetByPass ( QWidget *centralWidget );
    virtual void dbLoginByPass();
    virtual void showSettingsByPass();
    virtual void showHelpByPass( QString file );
    virtual void showStatusWidgetByPass ( QWidget *widget );
    virtual void removeStatusWidgetByPass ( QWidget *widget );
		
	private:
		QString myHelpPage;

  signals:
    void error ( const QString &string );
    void warning ( const QString &string );
    void information ( const QString &string );

    void addMenu ( QMenu* newMenu, const QString &parent );
    void delMenu ( QMenu *del );
    void addAction ( QAction* newMenu, const QString &parent );
    void delAction ( QAction *del );
    void addToolbar ( QToolBar* newbar );
    void delToolbar ( QToolBar* delBar );

    void addNewDockWidget ( QDockWidget *widget, const Qt::DockWidgetArea &area );
    void setCentralWidget ( QWidget *centralWidget );
    void removeDockWidget ( QDockWidget *widget );

    void dbLogin();

    void showSettings();
    void reloadSettings();

    void showHelp( QString file );

    void showStatusWidget ( QWidget* widget );
    void removeStatusWidget ( QWidget *widget );
};

#endif // DDIALOG_H
