/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "config.h"
#include "enumcheck.h"

#include "medium.h"

#include "mwidget.h"
#include "mdialog.h"
#include "msqldatabase.h"
#include "materialpersonsmodel.h"
#include "materialperson.h"
#include "editmanufacturers.h"
#include "editgenres.h"
#include "minmodel.h"
#include "mediummin.h"
#include "editmedium.h"

#include <QSqlDriver>
#include <QSqlQuery>
#include <QSqlError>
#include <QSqlRecord>
#include <QString>
#include <QVariant>
#include <QMessageBox>
#include <QBuffer>
#include <QByteArray>
#include <QImage>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QTextTable>
#include <QTextCursor>
#include <QStringListModel>

QString Medium::exportFormatToString( Medium::ExportFormat format)
{
	QMap<Medium::ExportFormat, QString> map;
	map.insert(Medium::Table, tr("Table"));
// 	map.insert(Medium::TablePerType, tr("One table per type"));
// 	map.insert(Medium::Long, tr("One page per medium"));
	return map.value(format, QString());
}

Medium::Medium ( Medium *newParent ) : MObject ( newParent )
{
	init(newParent);
}

Medium::Medium(QString name, QString group, Medium* newParent): MObject(newParent)
{
	myName = name;
	myGroup = group;
	init(newParent);
}

Medium::~Medium()
{
  qDeleteAll ( myChilds );
	myChilds.clear();
	delete matPersonModel;
}

Medium* Medium::newMedium()
{
	return new Medium();
}

QString Medium::name()
{
    return myName;
}

QString Medium::group()
{
    return myGroup;
}

bool Medium::select ( QSqlRecord record )
{
  if ( record.isEmpty() )
    return false;

  setId ( record.value ( "id" ).toUInt() );

  setType ( record.value ( "materialTypesId" ).toUInt() );

  setManufacturer ( record.value ( "manufacturersId" ).toUInt() );

  setGenre ( record.value ( "genresId" ).toUInt() );

  setTitle ( record.value ( "title" ).toString() );

  setListPrice ( record.value ( "listPrice" ).toDouble() );

  setListPriceCurrency ( record.value ( "listPriceCurrenciesId" ).toUInt() );

  setBoughtPrice ( record.value ( "boughtPrice" ).toDouble() );

  setBoughtPriceCurrency ( record.value ( "boughtPriceCurrenciesId" ).toUInt() );

  setEvaluation ( record.value ( "evaluation" ).toInt() );

  setWish ( record.value ( "wish" ).toBool() );
	
	setGiftOnly ( record.value ( "giftOnly" ).toBool() );
	
	setLiteratureOnly ( record.value ( "literatureOnly" ).toBool() );

  setImage ( QPixmap::fromImage( QImage::fromData ( record.value ( "image" ).toByteArray() ) ) );
	
	setDescription( record.value("description").toString() );
	
  setUser ( record.value ( "usersId" ).toUInt() );

	MSqlDatabase db = MSqlDatabase::database();
  bool closeDB = !db.isOpen();
  if ( closeDB )
		if( !db.open() )
			return false;

	bool result = true;
	if ( id() != 0 ){
		if(!loadKeywords())
			result = false;
    matPersonModel->loadAllPersons ( id() );
		setSource(Medium::Local);
		minModel->loadAllMINs( id() );
		setSourceIcon(QIcon(":icons/128x128/meganizer.png"));
	}

  if ( closeDB )
    db.close();
	
  return result;
}

bool Medium::select ( unsigned int searchId, unsigned int userId )
{
	MSqlDatabase db = MSqlDatabase::database();
  bool closeDB = !db.isOpen();
  if ( closeDB )
		if( !db.open() )
			return false;

  QString userCondition;

  QString queryString = QString ( "SELECT * FROM viewmaterials WHERE id='%1'" ).arg ( searchId );

  if ( userId == 0 ) {
    QSqlQuery queryUser ( QString ( "SELECT id FROM users WHERE userName='%1'" ).arg ( db.userName() ) );
    queryUser.exec();

    if ( !queryUser.first() ) {
      queryString.replace ( "viewmaterials", "materials" );
    }

    else {
      userId = queryUser.record().value ( "id" ).toUInt();
      queryString += QString ( " AND usersId='%1'" ).arg ( userId );
      setUser ( userId );
    }
  }


  QSqlQuery query;

  query.exec ( queryString );

  if ( !query.first() ) {
    if ( closeDB )
      db.close();

    return false;
  }

  select ( query.record() );

  if ( closeDB )
    db.close();

  if ( id() <= 0 )
    return false;

  return true;
}

bool Medium::store ( bool addOwner, bool startTransaction )
{
	// preparing needed values
	if(!prepareManufacturer()){
		emit error(tr("Preparation of manufacturer failed!"));
		return false;
	}
	if(!prepareGenres()){
		emit error(tr("Preparation of genre failed!"));
		return false;
	}
	
	// check values
	if(genre() < 1){
		emit error(tr("No valid genre given!"));
		return false;
	}
	if(manufacturer() < 1){
		emit error(tr("No valid manufacturer given!"));
		return false;
	}
	if(type() < 1){
		emit error(tr("No valid material type given!"));
		return false;
	}
	if(listPriceCurrency() < 1){
		emit error(tr("No valid list price currency given!"));
		return false;
	}
	if(boughtPriceCurrency() < 1){
		emit error(tr("No valid bought price currency given!"));
		return false;
	}
	
	// if startTransaction is false, a transaction is already started
	// so we should start one again
	bool transactions = false;
	if(startTransaction)
		transactions = transactionsSupported;
	MSqlDatabase db = MSqlDatabase::database();
  bool closeDB = !db.isOpen();
  if ( closeDB )
		if( !db.open() ){
			emit error(tr("Could not connect to database"));
			return false;
		}

	if(transactions)
		db.transaction();
		
  QSqlQuery query;

  if ( id() != 0 ) {
    // it's an update
    if ( !query.prepare ( "UPDATE materials SET "
                          "materialtypesId=:materialtypesId, manufacturersId=:manufacturersId, genresId=:genresId, "
                          "title=:title, listPrice=:listPrice, listPriceCurrenciesId=:listPriceCurrenciesId, "
                          "description=:description, image=:image "
                          "WHERE id=:id"
                        ) ) {
			emit error(tr("Error while storing medium!\n%1\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
			if(transactions)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
      if ( closeDB )
        db.close();

			return false;
    }
  }

  else {
    if ( !query.prepare ( "INSERT INTO materials "
                          "(materialtypesId, manufacturersId, genresId, "
                          " title, listPrice, listPriceCurrenciesId, "
                          " description, image)"
                          "VALUES (:materialtypesId, :manufacturersId, :genresId, "
                          "	      :title, :listPrice, :listPriceCurrenciesId, "
                          "        :description, :image)"
                        ) ) {
			emit error(tr("Error while storing medium!\n%1\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
			if(transactions)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
      if ( closeDB )
        db.close();

			return false;
    }
  }

	query.bindValue ( ":materialtypesId", type() );
  query.bindValue ( ":manufacturersId", manufacturer() );
  query.bindValue ( ":genresId", genre() );
  query.bindValue ( ":title", title() );
  query.bindValue ( ":listPrice", listPrice() );
  query.bindValue ( ":listPriceCurrenciesId", listPriceCurrency() );
  query.bindValue ( ":description", description() );
  query.bindValue ( ":image",  imageToByteArray());

  if ( id() != 0 )
    query.bindValue ( ":id", id() );


  if ( !query.exec ( ) ) {
		emit error(tr("Error while storing medium!\n%1\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
		if(transactions)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
    if ( closeDB )
      db.close();

    return false;
  }

  // set the users relation
  // therefore get own usersId
	query.prepare("SELECT id FROM users WHERE userName=:userName");
	query.bindValue(":userName", db.userName());
  query.exec();

  if ( !query.first() ) {
		emit error(tr("Error while storing medium!\n%1\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
		if(transactions)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
    if ( closeDB )
      db.close();

    return false;
  }

  unsigned int usersId = query.value ( 0 ).toUInt();
  if ( id() == 0 ) {
    // the id is null, so it is an insert, we than assume, that the medium belongs the user which
    // tries to insert it

    // first get the id of the material
		query.prepare("SELECT id FROM materials WHERE"
									" materialtypesId=:materialtypesId"
									" AND manufacturersId=:manufacturersId"
									" AND genresId=:genresId"
									" AND title=:title");
		query.bindValue(":materialtypesId", type());
		query.bindValue(":manufacturersId", manufacturer());
		query.bindValue(":genresId", genre());
		query.bindValue(":title", title());
    query.exec();
		if( query.size() != 1){
			emit error(tr("Error while storing medium!\nThe same medium seems to be already there!").arg(query.lastQuery()).arg(query.lastError().text()));
			if(transactions)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
			if ( closeDB )
        db.close();

      return false;
		}
			
    if ( !query.first() ) {
			emit error(tr("Error while storing medium!\n%1\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
			if(transactions)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
      if ( closeDB )
        db.close();

			return false;
    }
    setId ( query.value ( 0 ).toUInt() );

    // set user relation
		query.prepare("INSERT INTO userownmaterials (materialsId, usersId, boughtPrice,"
									"  boughtPriceCurrenciesId, evaluation, wish, giftOnly, literatureOnly)"
									" VALUES(:materialsId, :usersId, :boughtPrice, :boughtPriceCurrenciesId,"
									"  :evaluation, :wish, :giftOnly, :literatureOnly)");
		query.bindValue(":materialsId", id());
		query.bindValue(":usersId", usersId);
		query.bindValue(":boughtPrice", boughtPrice());
		query.bindValue(":boughtPriceCurrenciesId", boughtPriceCurrency());
		query.bindValue(":evaluation", evaluation());
		query.bindValue(":wish", wish());
		query.bindValue(":giftOnly", giftOnly());
		query.bindValue(":literatureOnly", literatureOnly());
    if ( !query.exec() ) {
			emit error(tr("Error while storing medium!\n%1\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
			if(transactions)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
      if ( closeDB )
        db.close();

      return false;
    }
		setUser(usersId);
  }
  else {
    // it's an update, so we must check if the user is already an owner of the medium
		query.prepare("SELECT id FROM userownmaterials WHERE materialsId=:materialsId AND usersId=:usersId");
		query.bindValue(":materialsId", id());
		query.bindValue(":usersId", usersId);
    query.exec();

    if ( query.first() ) {
      // the entry exists, so we update it
			query.prepare("UPDATE userownmaterials SET boughtPrice=:boughtPrice, boughtPriceCurrenciesId=:boughtPriceCurrenciesId, "
										" evaluation=:evaluation, wish=:wish, giftOnly=:giftOnly, literatureOnly=:literatureOnly"
										" WHERE materialsId=:materialsId AND usersId=:usersId");
			query.bindValue(":boughtPrice", boughtPrice());
			query.bindValue(":boughtPriceCurrenciesId", boughtPriceCurrency());
			query.bindValue(":evaluation", evaluation());
			query.bindValue(":wish", wish());
			query.bindValue(":giftOnly", giftOnly());
			query.bindValue(":literatureOnly", literatureOnly());
			query.bindValue(":materialsId", id());
			query.bindValue(":usersId", user());
      if ( !query.exec() ) {
				emit error(tr("Error while storing medium!\n%1\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
				if(transactions)
					if(!db.rollback())
						emit error(tr("Rolling back failed!"));
        if ( closeDB )
          db.close();

        return false;
      }

    }
    else if ( addOwner ) {
      // the entry doesn't exists, but we should add owner,
      // so we add a new entry to userownmaterials
			query.prepare("INSERT INTO userownmaterials (materialsId, usersId, boughtPrice,"
										"  boughtPriceCurrenciesId, evaluation, wish, giftOnly, literatureOnly)"
										" VALUES(:materialsId, :usersId, :boughtPrice, :boughtPriceCurrenciesId,"
										"  :evaluation, :wish, :giftOnly, :literatureOnly)");
			query.bindValue(":materialsId", id());
			query.bindValue(":usersId", usersId);
			query.bindValue(":boughtPrice", boughtPrice());
			query.bindValue(":boughtPriceCurrenciesId", boughtPriceCurrency());
			query.bindValue(":evaluation", evaluation());
			query.bindValue(":wish", wish());
			query.bindValue(":giftOnly", giftOnly());
			query.bindValue(":literatureOnly", literatureOnly());
      if ( !query.exec() ) {
				emit error(tr("Error while storing medium!\n%1\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
				if(transactions)
					if(!db.rollback())
						emit error(tr("Rolling back failed!"));
        if ( closeDB )
          db.close();

        return false;
      }
      setUser(usersId);
    }
  }

  if ( matPersonModel ) {
    if ( !matPersonModel->store ( id() ) ) {
			if(transactions)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
      if ( closeDB )
        db.close();
			emit error(tr("Could not store material person model!"));
      return false;
    }
  }
  
	if ( minModel ) {
    if ( !minModel->store ( id() ) ) {
			if(transactions)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
      if ( closeDB )
        db.close();
			emit error(tr("Could not store material person model!"));
      return false;
    }
  }

  if(!storeKeywords()){
		if(transactions)
			if(!db.rollback())
					emit error(tr("Rolling back failed!"));
      if ( closeDB )
        db.close();
			emit error(tr("Could not store material person model!"));
      return false;
  }
	
	if(transactions){
		if(!db.commit()){
			emit error(tr("Could not store medium! Commit has failed!"));
			if ( closeDB )
				db.close();
			return false;
		}
	}
	
  if ( closeDB )
    db.close();

  return true;
}

bool Medium::setValue ( QString attribute, QVariant value )
{
	if(attribute == "source"){
		return false;
	}
	
	if(attribute == "type"){
		bool check;
		value.toUInt(&check);
		if(!check)
			return false;
		setType(value.toUInt());
		return true;
	}
	
	if(attribute == "manufacturer"){
		bool check;
		value.toUInt(&check);
		if(!check)
			return false;
		setManufacturer ( value.toUInt() );
		return true;
	}

	if(attribute == "genre"){
		bool check;
		value.toUInt(&check);
		if(!check)
			return false;
		setGenre( value.toUInt() );
		return true;
	}

	if(attribute == "title"){
		setTitle ( value.toString() );
		return true;
	}

	if(attribute == "persons"){
		QStringList person = value.toStringList();
		if ( person.size() != 3 )
			return false;
		if ( person[0].isEmpty() )
			return false;
		MaterialPersonsModel *model = materialPersons();
		if ( !model )
			return false;

		bool check = false;
		unsigned int role = person[2].toUInt ( &check );
		// it's a id
		if ( check ) {
			model->addPerson ( person[0], person[1], role );
			return true;
		}
		model->addPerson ( person[0], person[1], person[2] );
		return true;
	}

  if(attribute == "min"){
		if(value.canConvert(QVariant::StringList)){
			if(value.toStringList().size() < 1){
				return false;
			}
			else if(value.toStringList().size() == 2){
				if(value.toStringList()[0].isEmpty())
					return false;
				setMIN(value.toStringList()[0], value.toStringList()[1]);
				return true;
			}
			else{
				if(value.toStringList()[0].isEmpty())
					return false;
				setMIN(value.toStringList()[0]);
				return true;
			}
		}
		if(value.toString().isEmpty())
			return false;
		setMIN ( value.toString() );
		return true;
	}

	if(attribute == "listPrice"){
		bool check;
		value.toDouble(&check);
		if(!check)
			return false;
		setListPrice ( value.toDouble() );
		return true;
	}

	if(attribute == "listPriceCurrency"){
		bool check;
		value.toUInt(&check);
		if(!check)
			return false;
		setListPriceCurrency ( value.toUInt() );
		return true;
	}

	if(attribute == "description"){
		setDescription(value.toString());
		return true;
	}

	if(attribute == "owner"){
		bool check;
		value.toUInt(&check);
		if(!check)
			return false;
		setUser ( value.toUInt() );
		return true;
	}
	return false;
}

bool Medium::setValue ( int attributeIndex, QVariant value )
{
	foreach(ColKeyTrans item, supportedAttrs()){
		if(item.column == attributeIndex)
			return setValue(item.key, value);
	}
	return false;
}

QVariant Medium::value ( QString attribute, int role )
{
	if(attribute ==  "type")
      return type();

	if(attribute ==  "manufacturer")
      return manufacturerName();

	if(attribute ==  "genre")
			return genreName();
			
	if(attribute ==  "title")
      return title();

	if(attribute ==  "persons")
			return materialPersonsCombined();
			
	if(attribute ==  "min")
      return minsCombined();

	if(attribute ==  "listPrice")
      return listPrice();

	if(attribute ==  "listPriceCurrency")
      return listPriceCurrency();

	if(attribute ==  "description")
			return description();
			
	if(attribute ==  "owner")
      return user();

  return QVariant();
}

QVariant Medium::value ( int attributeIndex, int role )
{
	if ( role == Qt::DecorationRole ) {
    if ( attributeIndex == attrs("source").column )
      return sourceIcon();

    return QVariant();
  }
  
	if ( role == Qt::ForegroundRole ) {
    if ( markForRemovel() )
      return QBrush ( QColor ( 200, 200, 200 ) );

    return QVariant();
	}

	foreach(ColKeyTrans item, supportedAttrs()){
		if(item.column == attributeIndex)
			return value(item.key, role);
	}
	return false;
}

QStringList Medium::supportedAttributes()
{
	QStringList result;
	foreach(ColKeyTrans item, supportedAttrs()){
		result << item.key;
	}
	return result;
}

QStringList Medium::supportedAttributesTrans()
{
	QStringList result;
	foreach(ColKeyTrans item, supportedAttrs()){
		result << item.translation;
	}
	return result;
}

QList<ColKeyTrans> Medium::supportedAttrs()
{
	QList<ColKeyTrans> attrs;
	attrs << ColKeyTrans(0, "source", tr ( "Source" ));
	attrs << ColKeyTrans(1, "type", tr ( "Type" ));
	attrs << ColKeyTrans(2, "manufacturer", tr ( "Manufacturer" ));
	attrs << ColKeyTrans(3, "genre", tr ( "Genre" ));
	attrs << ColKeyTrans(4, "title", tr("Title"));
	attrs << ColKeyTrans(5, "persons", tr ( "Persons" ));
  attrs << ColKeyTrans(6, "min", tr ( "MIN" ));
	attrs << ColKeyTrans(7, "listPrice", tr ( "List Price" ));
  attrs << ColKeyTrans(8, "listPriceCurrency", tr ( "Currency" ));
	attrs << ColKeyTrans(9, "description", tr ( "Description" ));
	attrs << ColKeyTrans(10, "owner", tr ( "Owner" ));

  return attrs;
}

ColKeyTrans Medium::attrs(QString key, int newCol)
{
	QList<ColKeyTrans> items;
	if(newCol > -1){
		items = Medium::supportedAttrs();
	}
	else{
		items = supportedAttrs();
	}
		
	foreach(ColKeyTrans item, items){
		if(item.key == key){
			if(newCol > -1)
				item.column = newCol;
			return item;
		}
	}
	return ColKeyTrans();
}

EditMedium *Medium::editWidget(MWidget* parent)
{
  return new EditMedium(parent);
}

bool Medium::edit(int editMode, MWidget* parent)
{
	EditMedium::EditMode editMode_;
  editMode_ = check::bounds(EditMedium::Edit, editMode, EditMedium::New);
	
	// if medium is already in database and the EditMode is Add,
	// the only thing we must do is, to store it with current user as owner
	if((editMode_ == EditMedium::Add) && (id() > 0))
		return store(true);
	
	if(editMode_ == EditMedium::New)
		return false;
	
	EditMedium *widget = this->editWidget(parent);
  if ( !widget ) {
    emit error ( tr ( "Could no get edit wigdet" ) );
    return false;
  }
  if(!parent){
		// here only a few features are handled
		connectByPass ( widget );
	}
	else{
		// in this case, all features are handled
		parent->connectByPass(widget);
	}
	
	switch(editMode_){
		case EditMedium::Add:
			widget->setAddMode(this);
			break;
		case EditMedium::Edit:
			widget->setEditMode(this);
			break;
		default:
			delete widget;
			return false;
	}
	
	bool result = false;
	if ( widget->exec() == QDialog::Accepted ){
		result = true;
		if(editMode == EditMedium::Add){
			setSource(Medium::Local);
			setSourceIcon(QIcon ( ":icons/128x128/meganizer.png" ));
		}
	}
		
	delete widget;
	
	return result;
}

bool Medium::isValid(){
  if(title().isEmpty())
    return false;
  return true;
}

Medium* Medium::copyMedium()
{
	Medium *newMed = newMedium();
	newMed->setId(id());
	newMed->setTitle(title());
	newMed->setImage(image());
	newMed->setMIN(minModel->copyModel());
	newMed->setManufacturer(manufacturer(), false);
	newMed->setManufacturer(manufacturerName(), false);
	newMed->setGenre(genre(), false);
	newMed->setGenre(genreName(), false);
	newMed->setDescription(description());
	newMed->setBoughtPrice(boughtPrice());
	newMed->setBoughtPriceCurrency(boughtPriceCurrency());
	newMed->setListPrice(listPrice());
	newMed->setListPriceCurrency(listPriceCurrency());
	newMed->setEvaluation(evaluation());
	newMed->setUser(user());
	newMed->setType(type());
	newMed->setWish(wish());
	newMed->setGiftOnly(giftOnly());
	newMed->setLiteratureOnly(literatureOnly());
	newMed->setSourceIcon(sourceIcon());
	newMed->setMaterialPersons(materialPersons()->copyModel());
	foreach(QString keyword, keywordList)
		newMed->addKeyword(keyword);
	
	return newMed;
}

bool Medium::exportMedium(QTextTable* table, Medium::ExportFormat format, bool titleOnly)
{
	if(!table)
		return false;
	
	// collecting the data which should be inserted	
	QStringList values;
	if(titleOnly){
		values << tr("Title");
		values << tr("MIN's");
		values << tr("Wish");
	}
	else{
		values << title();
		values << minsCombined();
		if(wish())
			values << tr("Yes");
	}
	
	// insert the place at the table we need
	if(!titleOnly){
		// the first row is present at the table, because empty table can not be created
#if QT_VERSION_MINOR < 5
		table->insertRows(table->rows(), 1);
#else
		table->appendRows(1);
#endif
	}
	if(table->columns() < values.size()){
#if QT_VERSION_MINOR < 5
		table->insertColumns(table->columns(), values.size() - table->columns());
#else
		table->appendColumns(values.size() - table->columns());
#endif
	}
	int row = table->rows() - 1;
	bool result = true;
	for(int i = 0; i < values.size(); i++){
		QTextTableCell cell = table->cellAt(row, i);
		QTextCursor cursor = cell.firstCursorPosition();
		cursor.insertText(values[i]);
	}
	return result;
}

void Medium::setParent ( Medium *newParent )
{
  myParent = newParent;
  emit changed(this);
}

Medium *Medium::parent() const
{
  return myParent;
}

void Medium::setId ( unsigned int newId )
{
  myId = newId;
  emit changed(this);
}

unsigned int Medium::id() const
{
  return myId;
}

void Medium::setType ( unsigned int typeId )
{
  myTypeId = typeId;
	myTypeName = QString();
	if(typeId > 0){
		MSqlDatabase db = MSqlDatabase::database();
		bool closeDB = !db.isOpen();
		if(closeDB)
			db.open();
		if( db.isOpen() ){
			QSqlQuery query;
			query.exec( QString("SELECT name FROM materialtypes WHERE id=%1").arg(typeId));
			if(query.first())
				myTypeName = query.record().value("name").toString();
		}
		if(closeDB)
			db.close();
	}
  emit changed(this);
}

unsigned int Medium::type() const
{
  return myTypeId;
}

QString Medium::typeName() const
{
	return myTypeName;
}

void Medium::setManufacturer ( unsigned int manuId, bool setName )
{
  myManuId = manuId;
	
	// fetch name of manufacturer
	if(setName){
		setManufacturer(QString(), false);
		if(manuId > 0){
			MSqlDatabase db = MSqlDatabase::database();
			bool closeDB = !db.isOpen();
			if(closeDB)
				db.open();
			if( db.isOpen() ){
				QSqlQuery query;
				query.exec( QString("SELECT name FROM manufacturers WHERE id=%1").arg(manuId));
				if(query.first())
					// don't let setManufacturer set the id afterwards (loop)
					setManufacturer(query.record().value("name").toString(), false);
			}
			if(closeDB)
				db.close();
		}
	}
  emit changed(this);
}

void Medium::setManufacturer ( QString name, bool setId )
{
	myManuName = name.trimmed();
	if(setId){
		setManufacturer(0, false);
		if(!myManuName.isEmpty()){
			MSqlDatabase db = MSqlDatabase::database();
			bool closeDB = !db.isOpen();
			if(closeDB)
				db.open();
			if( db.isOpen() ){
				QSqlQuery query;
				query.prepare("SELECT id FROM manufacturers WHERE name=:name");
				query.bindValue(":name", name);
				query.exec();
				// we only set the id, if the relation is unique
				if(query.size() == 1){
					if(query.first())
						// don't let setManufacturer set the name afterwards (loop)
						setManufacturer(query.record().value("id").toUInt(), false);
				}
			}
			if(closeDB)
				db.close();
		}
	}
	emit changed(this);
}

unsigned int Medium::manufacturer() const
{
  return myManuId;
}

QString Medium::manufacturerName() const
{
	return myManuName;
}

bool Medium::prepareManufacturer()
{
	if(manufacturer() > 0)
		return true;
	
	if(!manufacturerName().isEmpty()){
		// so try to get it from database
		setManufacturer(manufacturerName());
		if(manufacturer() > 0)
			return true;
	}
	
	// use must add a new one
	EditManufacturers editManu(manufacturerName());
	if(editManu.exec() != QDialog::Accepted)
		return false;
	
	if(editManu.selectedId() < 1)
		return false;
	
	setManufacturer(editManu.selectedId(), false);
	setManufacturer(editManu.selectedName(), false);
	return true;
}

void Medium::setGenre ( unsigned int genreId, bool setName )
{
  myGenreId = genreId;
	
	// fetch name of the genre
	if(setName){
		setGenre(QString(), false);
		if(genreId > 0){
			MSqlDatabase db = MSqlDatabase::database();
			bool closeDB = !db.isOpen();
			if(closeDB)
				db.open();
			if( db.isOpen() ){
				QSqlQuery query;
				query.exec( QString("SELECT name FROM genres WHERE id=%1").arg(genreId));
				if(query.first())
					// don't let setGenre set the id afterwards (loop)
					setGenre(query.record().value("name").toString(), false);
			}
			if(closeDB)
				db.close();
		}
	}
  emit changed(this);
}

void Medium::setGenre(QString name, bool setId)
{
	myGenreName = name.trimmed();
	
	// fetch name of the genre
	if(setId){
		setGenre(0, false);
		if(!myGenreName.isEmpty()){
			MSqlDatabase db = MSqlDatabase::database();
			bool closeDB = !db.isOpen();
			if(closeDB)
				db.open();
			if( db.isOpen() ){
				QSqlQuery query;
				query.prepare("SELECT id FROM genres WHERE name=:name");
				query.bindValue(":name", name);
				query.exec();
				if(query.size() == 1){
					if(query.first())
						// don't let setGenre set the name afterwards (loop)
						setGenre(query.record().value("id").toUInt(), false);
				}
			}
			if(closeDB)
				db.close();
		}
	}
  emit changed(this);
}

unsigned int Medium::genre() const
{
  return myGenreId;
}

QString Medium::genreName() const
{
	return myGenreName;
}

bool Medium::prepareGenres()
{
	if(genre() > 0)
		return true;
	
	if(!genreName().isEmpty()){
		// so try to get it from database
		setGenre(genreName());
		if(genre() > 0)
			return true;
	}
	
	// use must add a new one
	EditGenres editGenre(genreName());
	if(editGenre.exec() != QDialog::Accepted)
		return false;
	
	if(editGenre.selectedId() < 1)
		return false;
	
	setGenre(editGenre.selectedId(), false);
	setGenre(editGenre.selectedName(), false);
	return true;
}

void Medium::setTitle ( QString newTitle )
{
	// use only the number of characters, which can be stored
  myTitle = newTitle.left(120);
	myTitle = myTitle.trimmed();
  emit changed(this);
}

QString Medium::title() const
{
  return myTitle;
}

void Medium::setMIN(QString newMin, QString minType)
{
	minModel->addMIN(newMin, minType);
	emit changed(this);
}

void Medium::setMIN(QString newMin, unsigned int minTypeId)
{
	minModel->addMIN(newMin, minTypeId);
	emit changed(this);
}

void Medium::setMIN(MINModel* newMINModel)
{
	delete minModel;
	minModel = newMINModel;
	emit changed(this);
}

MINModel* Medium::mins() const
{
	return minModel;
}

QString Medium::minsCombined() const
{
	return minModel->combineMINs();
}

QString Medium::min(QStringList minTypes) const
{
	foreach(QString minType, minTypes){
		QStringList minList = minModel->mins(minType);
		if(minList.size() > 0)
			return minList[0];
	}
	return QString();
}

QString Medium::min(QString minType) const
{
	return min(QStringList(minType));
}

void Medium::setListPrice ( double price )
{
  myListPrice = price;
  emit changed(this);
}

double Medium::listPrice() const
{
  return myListPrice;
}

void Medium::setListPriceCurrency ( unsigned int curId )
{
  myCurId = curId;
  emit changed(this);
}

void Medium::setListPriceCurrency ( QString shortCode )
{
	if(shortCode.isEmpty())
		return;
	
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if( db.isOpen() ){
		QSqlQuery query;
		query.prepare("SELECT id FROM currencies WHERE shortName=:shortName OR name=:name");
		query.bindValue(":shortName", shortCode);
		query.bindValue(":name", shortCode);
		query.exec();
		if(query.size() == 1){
			if(query.first()){
				setListPriceCurrency(query.record().value("id").toUInt());
				emit changed(this);
			}
		}
	}
	if(closeDB)
		db.close();
}

unsigned int Medium::listPriceCurrency() const
{
  return myCurId;
}

void Medium::setDescription ( QString desc )
{
	// strip out html tags
	desc.replace( QRegExp("<[^>]*>"), "" );
  myDescription = desc.trimmed();
  emit changed(this);
}

QString Medium::description() const
{
  return myDescription;
}

void Medium::setImage ( QPixmap newImage )
{
  myImage = newImage;
  emit changed(this);
}

void Medium::setImage(QString url)
{
	QNetworkAccessManager *netMan = new QNetworkAccessManager(this);
	reply = netMan->get(QNetworkRequest(QUrl(url)));
  connect(reply, SIGNAL(finished()),
      this, SLOT(imageDownloaded()));
}

QPixmap Medium::image() const
{
  return myImage;
}

QByteArray Medium::imageToByteArray()
{
	QByteArray bytes;
	QBuffer buffer(&bytes);
	buffer.open(QIODevice::WriteOnly);
	myImage.save(&buffer, "JPG");
	return bytes;
}

void Medium::setUser ( unsigned int userId )
{
  myUserId = userId;
  emit changed(this);
}

unsigned int Medium::user() const
{
  return myUserId;
}

void Medium::setBoughtPrice ( double price )
{
  myBoughtPrice = price;
  emit changed(this);
}

double Medium::boughtPrice() const
{
  return myBoughtPrice;
}

void Medium::setBoughtPriceCurrency ( unsigned int curId )
{
	myBoughtCurId = curId;
  emit changed(this);
}

void Medium::setBoughtPriceCurrency ( QString shortCode )
{
	if(shortCode.isEmpty())
		return;
	
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if( db.isOpen() ){
		QSqlQuery query;
		query.prepare("SELECT id FROM currencies WHERE shortName=:shortName OR name=:name");
		query.bindValue(":shortName", shortCode);
		query.bindValue(":name", shortCode);
		query.exec();
		if(query.size() == 1){
			if(query.first()){
				setBoughtPriceCurrency(query.record().value("id").toUInt());
				emit changed(this);
			}
		}
	}
	if(closeDB)
		db.close();
}

unsigned int Medium::boughtPriceCurrency() const
{
  return myBoughtCurId;
}

void Medium::setEvaluation ( int eval )
{
  myEval = eval;
  emit changed(this);
}

int Medium::evaluation() const
{
  return myEval;
}

void Medium::setWish ( bool isWish )
{
  myWish = isWish;
  emit changed(this);
}

bool Medium::wish() const
{
  return myWish;
}

void Medium::setLiteratureOnly(bool isLiterature)
{
	myLiterature = isLiterature;
}

bool Medium::literatureOnly() const
{
	return myLiterature;
}

void Medium::setGiftOnly(bool isGift)
{
	myGift = isGift;
}

bool Medium::giftOnly() const
{
	return myGift;
}

void Medium::setSourceIcon(QIcon icon)
{
	mySourceIcon = icon;
	emit changed(this);
}

QIcon Medium::sourceIcon() const
{
	return mySourceIcon;
}

void Medium::setSource(Medium::Origin newOrigin)
{
	myOrigin = newOrigin;
}

Medium::Origin Medium::source() const
{
	return myOrigin;
}

void Medium::setMaterialPerson(QString name, QString givenName, QString role)
{
	materialPersons()->addPerson(name, givenName, role);
}

void Medium::setMaterialPerson(QString name, QString givenName, unsigned int role)
{
	materialPersons()->addPerson(name, givenName, role);
}

void Medium::setMaterialPersons(MaterialPersonsModel* personModel)
{
	matPersonModel = personModel;
}

MaterialPersonsModel *Medium::materialPersons() const
{
  return matPersonModel;
}

QString Medium::materialPersonsCombined() const
{
	MaterialPersonsModel *model = materialPersons();
	if ( !model )
		return QString();
	return model->combinePersons();
}

bool Medium::removeMaterialPerson ( QModelIndex index )
{
  if ( !matPersonModel )
    return false;

  MaterialPerson *person = qobject_cast<MaterialPerson*>(matPersonModel->personFromIndex ( index ));

  if ( !person )
    return false;

  person->setMarkForRemovel();

  return true;
}

void Medium::setMarkForRemovel()
{
  shouldBeRemoved = true;
  emit changed(this);
}

bool Medium::markForRemovel()
{
  return shouldBeRemoved;
}

void Medium::addKeyword(QString keyword)
{
	// check wether keyword is already present
	if(keywordList.contains(keyword, Qt::CaseInsensitive))
		return;

	keywordList << keyword;
}

bool Medium::removeKeyword(QString keyword)
{
	if(!keywordList.contains(keyword, Qt::CaseInsensitive))
		return false;
	// we must do it ourself, because we are case insensitive
	// and QList::removeAll() is not case insensitive
	for(int i = 0; i < keywordList.size(); i++){
		if(keywordList[i].toUpper() == keyword.toUpper())
			if(keywordList.removeAll(keywordList[i]) < 0)
				return false;
	}
	return true;
}

QStringListModel* Medium::keywords()
{
	return new QStringListModel(keywordList);
}

void Medium::setChild ( Medium *newChild )
{
  myChilds.append ( newChild );
  emit changed(this);
}

QList<Medium*> Medium::childs() const
{
  return myChilds;
}

int Medium::rowCount() const
{
  return myChilds.count();
}

int Medium::columnCount()
{
  return supportedAttributes().size();
}


bool Medium::hasChildren() const
{
  if ( rowCount() > 0 )
    return true;

  return false;
}

void Medium::setCompatibilityMode(bool enable)
{
	myCompatibilityMode = enable;
}

bool Medium::operator== (const Medium &med) const
{
  if ( user() != med.user() )
    return false;

  if ( ( id() == 0 ) && ( med.id() == 0 ) ) {
		MINModel *maxMinModel = mins();
		MINModel *otherMinModel = med.mins();
		if(maxMinModel->rowCount() < med.mins()->rowCount()){
			maxMinModel = otherMinModel;
			otherMinModel = mins();
		}
		for(int i = 0; i < maxMinModel->rowCount(); i++){
			MediumMIN *actMin = maxMinModel->minFromIndex(maxMinModel->index(i, 0));
			if(actMin){
				if(actMin == otherMinModel->min(actMin->typeName()))
					return true;
			}
		}
    return false;
  }

  if ( id() == med.id() )
    return true;

  return false;
}

bool Medium::operator!=(const Medium& med) const
{
	return !operator==(med);
}

bool Medium::compatibilityMode() const
{
	return myCompatibilityMode;
}

void Medium::imageDownloaded()
{
	if(!reply)
		return;
	
  if(reply->error() != QNetworkReply::NoError){
// 		emit information(tr("Could not fetch image from url: %1. Error: %2").arg(reply->url().toString()).arg(reply->errorString()));
		return;
	}
		
	QByteArray data = reply->readAll();
	QPixmap imageData;
	if(!imageData.loadFromData(data))
		return;
	setImage(imageData);
	
	reply->deleteLater();
}

void Medium::init(Medium* newParent)
{
	myCompatibilityMode = false;
	setSource(Medium::Manual);
	reply = 0;
  setParent ( newParent );
  myId = 0;
	myTypeId = 0;
  myManuId = 0;
  myGenreId = 0;
  myListPrice = 0;
  myCurId = 1;
  myEval = 0;
  myUserId = 0;
  myBoughtPrice = 0;
  myBoughtCurId = 1;
  myEval = 0;
  myWish = false;
	myLiterature = false;
	myGift = false;
  matPersonModel = new MaterialPersonsModel(this);
	minModel = new MINModel(this);
  shouldBeRemoved = false;

  MSqlDatabase db = MSqlDatabase::database();
	transactionsSupported = db.driver()->hasFeature(QSqlDriver::Transactions);
}

bool Medium::loadKeywords()
{
	if(id() == 0)
		return false;

	QSqlQuery query;
	query.prepare("SELECT keyword FROM viewmaterialskeywords  WHERE materialsId=:materialsId ORDER BY keyword");
	query.bindValue(":materialsId", id());
	if(!query.exec())
		return false;
	while(query.next())
		keywordList << query.record().value("keyword").toString();
	return true;
}

bool Medium::storeKeywords()
{
	if(id() == 0)
		return false;
	
	// first, delete all the connections at the database,
	// where the keyword isn't in the list anymore
	QSqlQuery query;
	query.prepare("SELECT keywordsId, UPPER(keyword) FROM viewmaterialskeywords  WHERE materialsId=:materialsId ORDER BY keyword");
	query.bindValue(":materialsId", id());
	query.exec();
	while(query.next()){
		if(!keywordList.contains(query.record().value("keyword").toString(), Qt::CaseInsensitive)){
			// it's not in the list anymore, so remove the connection from the database
			QSqlQuery delQuery;
			delQuery.prepare("DELETE FROM materialhaskeywords WHERE materialsId=:materialsId AND UPPER(keywordsId)=UPPER(:keywordsId)");
			delQuery.bindValue(":materialsId", id());
			delQuery.bindValue(":keywordsId", query.record().value("keywordsId").toUInt());
			if(!delQuery.exec())
				return false;
		}
	}
	// add remaining keywords
	foreach(QString keyword, keywordList){
		query.prepare("SELECT id FROM viewmaterialskeywords WHERE materialsId=:materialsId AND UPPER(keyword)=UPPER(:keyword)");
		query.bindValue(":materialsId", id());
		query.bindValue(":keyword", keyword);
		if(!query.exec())
			return false;
		if(query.size() > 0)
			continue;

		// combination is not present, therefore we must add the new relation
		// to do this, we need to know the id of the keyword
		// and if the keyword isn't at the database already, we must add it first
		query.prepare("SELECT id FROM keywords WHERE UPPER(keyword)=UPPER(:keyword)");
		query.bindValue(":keyword", keyword);
		if(!query.exec())
			return false;
		if(query.size() == 0){
			// we must add it first
			query.prepare("INSERT INTO keywords (keyword) VALUES(:keyword)");
			query.bindValue(":keyword", keyword);
			if(!query.exec())
				return false;
			query.prepare("SELECT id FROM keywords WHERE keyword=:keyword");
			query.bindValue(":keyword", keyword);
			if(!query.exec())
				return false;
		}
		query.first();
		unsigned int keywordId = query.record().value("id").toUInt();
		if(keywordId == 0)
			return false;

		query.prepare("INSERT INTO materialhaskeywords (materialsId, keywordsId) VALUES(:materialsId, :keywordsId)");
		query.bindValue(":materialsId", id());
		query.bindValue(":keywordsId", keywordId);
		if(!query.exec())
			return false;
	}
	return true;
}

#include "medium.moc"
