/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumgenrewidget.h"

#include "medium.h"
#include "editgenres.h"
#include "searchbutton.h"
#include "msqldatabase.h"

#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QSqlQueryModel>
#include <QCompleter>

MediumGenreWidget::MediumGenreWidget(Medium* medium, QWidget* parent, Qt::WindowFlags flags)
	: MWidget(parent, flags)
{
	setupUi();
	setMedium(medium);
}

MediumGenreWidget::MediumGenreWidget(QWidget* parent, Qt::WindowFlags flags)
	: MWidget(parent, flags)
{
	setupUi();
	setMedium(0);
}

void MediumGenreWidget::setMedium(Medium* medium)
{
	if(this->medium)
		medium->disconnect(this);
	this->medium = medium;
	
	bool enable = false;
	if(medium){
		enable = true;
// 		connect(medium, SIGNAL(changed()), this, SLOT(setGenreFromMedium()));
	}
	setEnabled(enable);
	
	setGenreFromMedium();
}

void MediumGenreWidget::setGenre()
{
	if(!medium)
		return;
	
	medium->setGenre(genreEdit->text());
}

void MediumGenreWidget::setGenreFromMedium()
{
	if(medium)
		genreEdit->setText(medium->genreName());
}

void MediumGenreWidget::searchGenre()
{
	if(!medium)
		return;
	
  EditGenres *editGenres = new EditGenres( genreEdit->text(), this );
  connectByPass ( editGenres );

  if ( editGenres->exec() != QDialog::Accepted ){
		delete editGenres;
		return;
	}

	loadCompleter();
	
	medium->setGenre(editGenres->selectedId(), false);
	medium->setGenre(editGenres->selectedName(), false);
	genreEdit->setText(medium->genreName());
	delete editGenres;
}

void MediumGenreWidget::setupUi()
{
	this->medium = 0;
	complModel = 0;
	
	QHBoxLayout *mainLayout = new QHBoxLayout(this);
	setLayout(mainLayout);
	mainLayout->setMargin(0);
	
	QLabel *label = new QLabel(tr("Genre:"), this);
	mainLayout->addWidget(label);
	label->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	label->setMinimumSize(150, 23);
	
	genreEdit = new QLineEdit(this);
	mainLayout->addWidget(genreEdit);
	connect(genreEdit, SIGNAL(editingFinished()), this, SLOT(setGenre()));
	genreEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	genreEdit->setMinimumSize(150, 23);
	genreEdit->setMaximumHeight(30);
	
	SearchButton *button = new SearchButton(this);
	mainLayout->addWidget(button);
	connect(button, SIGNAL(clicked()), this, SLOT(searchGenre()));
	button->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	button->setMinimumSize(100, 23);
	button->setMaximumHeight(30);
	
	loadCompleter();
}

void MediumGenreWidget::loadCompleter()
{
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB){
		if(!db.open())
			return;
	}
		
	QString query = "SELECT name FROM genres ORDER BY name";
	if(!complModel){
		complModel = new QSqlQueryModel ( this );
		QCompleter *completer = new QCompleter ( complModel, this );
		completer->setCaseSensitivity ( Qt::CaseInsensitive );
		completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
		genreEdit->setCompleter ( completer );
	}
	complModel->setQuery ( query );
	if(closeDB)
		db.close();
}

#include "mediumgenrewidget.moc"
