/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "config.h"

#include "mediumimagewidget.h"

#include "medium.h"
#include "scaledimage.h"
#include "searchbutton.h"

#include <QSettings>
#include <QVBoxLayout>
#include <QHBoxLayout>
#ifdef KDE
	#include <KFileDialog>
	#include <KUrl>
#else
	#include <QFileDialog>
#endif

MediumImageWidget::MediumImageWidget(Medium* medium, QWidget* parent, Qt::WindowFlags flags)
	: MWidget(parent, flags)
{
	setupUi();
	setMedium(medium);
}

MediumImageWidget::MediumImageWidget(QWidget* parent, Qt::WindowFlags flags)
	: MWidget(parent, flags)
{
	setupUi();
	setMedium(0);
}

void MediumImageWidget::setMedium(Medium* medium)
{
	if(this->medium)
		medium->disconnect(this);
	this->medium = medium;
	
// 	if(medium)
// 		connect(medium, SIGNAL(changed()), this, SLOT(setImageFromMedium()));
	
	bool enable = true;
	if(!medium)
		enable = false;
	setEnabled(enable);
	setImageFromMedium();
}

void MediumImageWidget::setImage()
{
	if(!medium)
		return;
	
	medium->setImage(scaledImage->image());
}

void MediumImageWidget::setImageFromMedium()
{
	if(medium)
		scaledImage->setImage(medium->image());
}

void MediumImageWidget::searchImage()
{
	if(!medium)
		return;


  QStringList filters;
#ifdef KDE
	QString defaultFilter = QString("*.jpg *.png|%1").arg(tr("Images"));
#else
	QString defaultFilter = QString("%1 (*.jpg *.png)").arg(tr("Images"));
#endif
filters << defaultFilter;
#ifdef KDE
	filters << QString("*.*|%1").arg(tr("All Files"));
#else
	filters << QString("%1 (*.*)").arg(tr("All Files"));
#endif
	
	QSettings settings;
	QString lastDir = settings.value("lastExportDir", "").toString();
	QString wTitle = tr("Search image");
#ifdef KDE
	QString result = KFileDialog::getOpenFileName(KUrl(lastDir), filters.join("\n"), this, wTitle);
#else
	QString result = QFileDialog::getOpenFileName(this, wTitle, lastDir, filters.join(";;"), &defaultFilter);
#endif
	
	
  if(result.isEmpty())
    return;
  QPixmap image(result);
  if(image.isNull()){
    emit error(tr("Could not load selected image"));
    return;
  }
  scaledImage->setImage ( image );
	setImage();
}

void MediumImageWidget::setupUi()
{
	this->medium = 0;
	QVBoxLayout *mainLayout = new QVBoxLayout(this);
	setLayout(mainLayout);
	mainLayout->setMargin(0);
	
	scaledImage = new ScaledImage(this);
	mainLayout->addWidget(scaledImage);
// 	scaledImage->setMinimumSize(100, 100);
	
	QHBoxLayout *buttonLayout = new QHBoxLayout();
	mainLayout->addLayout(buttonLayout);
	buttonLayout->addStretch();
	
	SearchButton *button = new SearchButton(this);
	buttonLayout->addWidget(button);
	connect(button, SIGNAL(clicked()), this, SLOT(searchImage()));
	button->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	button->setMaximumHeight(30);
}

#include "mediumimagewidget.moc"
