/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "mediummanufacturerwidget.h"

#include "medium.h"
#include "editmanufacturers.h"
#include "searchbutton.h"
#include "msqldatabase.h"

#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QSqlQueryModel>
#include <QCompleter>

MediumManufacturerWidget::MediumManufacturerWidget(Medium* medium, QWidget* parent, Qt::WindowFlags flags)
	: MWidget(parent, flags)
{
	setupUi();
	setMedium(medium);
}

MediumManufacturerWidget::MediumManufacturerWidget(QWidget* parent, Qt::WindowFlags flags)
	: MWidget(parent, flags)
{
	setupUi();
	setMedium(0);
}

void MediumManufacturerWidget::setMedium(Medium* medium)
{
	if(this->medium)
		medium->disconnect(this);
	this->medium = medium;
	
// 	if(medium)
// 		connect(medium, SIGNAL(changed()), this, SLOT(setManufacturerFromMedium()));
		
	
	bool enable = true;
	if(!medium)
		enable = false;
	setEnabled(enable);
	
	setManufacturerFromMedium();
}

void MediumManufacturerWidget::setLabelText(QString text)
{
	manuLabel->setText(text);
}

QString MediumManufacturerWidget::labelText() const
{
	return manuLabel->text();
}

void MediumManufacturerWidget::setManufacturer()
{
	if(!medium)
		return;
	
	medium->setManufacturer(manuEdit->text());
}

void MediumManufacturerWidget::setManufacturerFromMedium()
{
	if(medium)
		manuEdit->setText(medium->manufacturerName());
}

void MediumManufacturerWidget::searchManufacturer()
{
	if(!medium)
		return;
	
  EditManufacturers *editManu = new EditManufacturers( manuEdit->text(), this );
  connectByPass ( editManu );

  if ( editManu->exec() != QDialog::Accepted )
		return;

	loadCompleter();
	
	medium->setManufacturer(editManu->selectedId(), false);
	medium->setManufacturer(editManu->selectedName(), false);
	manuEdit->setText(medium->manufacturerName());
}

void MediumManufacturerWidget::setupUi()
{
	this->medium = 0;
	complModel = 0;
	
	QHBoxLayout *mainLayout = new QHBoxLayout(this);
	setLayout(mainLayout);
	mainLayout->setMargin(0);
	
	manuLabel = new QLabel(tr("Manufacturer:"), this);
	mainLayout->addWidget(manuLabel);
	manuLabel->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	manuLabel->setMinimumSize(150, 23);
	
	manuEdit = new QLineEdit(this);
	mainLayout->addWidget(manuEdit);
	connect(manuEdit, SIGNAL(editingFinished()), this, SLOT(setManufacturer()));
	manuEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	manuEdit->setMaximumHeight(30);
	
	SearchButton *button = new SearchButton(this);
	mainLayout->addWidget(button);
	connect(button, SIGNAL(clicked()), this, SLOT(searchManufacturer()));
	button->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	button->setMinimumSize(100, 23);
	button->setMaximumHeight(30);
	
	loadCompleter();
}

void MediumManufacturerWidget::loadCompleter()
{
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB){
		if(!db.open())
			return;
	}
		
	QString query = "SELECT name FROM manufacturers ORDER BY name";
	if(!complModel){
		complModel = new QSqlQueryModel ( this );
		QCompleter *completer = new QCompleter ( complModel, this );
		completer->setCaseSensitivity ( Qt::CaseInsensitive );
		completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
		manuEdit->setCompleter ( completer );
	}
	complModel->setQuery ( query );
	if(closeDB)
		db.close();
}


#include "mediummanufacturerwidget.moc"
