/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "config.h"

#include "mediumownerwidget.h"

#include "personsmodel.h"
#include "medium.h"
#include "person.h"
#include "msqldatabase.h"

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlDriver>

MediumOwnerWidget::MediumOwnerWidget ( Medium* newMedium, QWidget* widget, Qt::WindowFlags flags )
	: MediumEditWidget ( widget, flags )
{
	init();
	setMedium(newMedium);
}

MediumOwnerWidget::MediumOwnerWidget ( QWidget* widget, Qt::WindowFlags flags )
	: MediumEditWidget ( widget, flags )
{
	init();
}

void MediumOwnerWidget::setMedium ( Medium* newMedium )
{
	if(medium)
		medium->disconnect(this);
	medium = newMedium;
	
	ownerSelected(QModelIndex());
	userSelected(QModelIndex());
	ownerView->setModel(0);
	userView->setModel(0);
	delete ownerModel;
	delete userModel;
	ownerModel = 0;
	userModel = 0;
	
	if(!medium)
		return;
	
	ownerModel = new PersonsModel(this);
	userModel = new PersonsModel(this);
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();

	QSqlQuery query;
	// find owner
	QList<int> presentIds;
	if(!query.exec(QString("SELECT * FROM userownmaterials WHERE materialsId=%1").arg(medium->id()))){
		if(closeDB)
			db.close();
		return;
	}
	while(query.next()){
		presentIds << query.record().value("usersId").toUInt();
	}
	
	query.exec("SELECT * FROM users");
	while(query.next()){
		unsigned int userId = query.record().value("id").toUInt();
		if(presentIds.indexOf(userId) != -1){
			ownerModel->addPerson(
				query.record().value("id").toUInt(),
				query.record().value("name").toString(),
				query.record().value("givenName").toString()
			);
		}
		else{
			userModel->addPerson(
				query.record().value("id").toUInt(),
				query.record().value("name").toString(),
				query.record().value("givenName").toString()
			);
		}
	}
	if(closeDB)
		db.close();

	ownerView->setModel(ownerModel);
	ownerModel->hideColumns(ownerView);
	userView->setModel(userModel);
	userModel->hideColumns(userView);
}

bool MediumOwnerWidget::store(bool useOldTransaction)
{
	if(!ownerModel)
		return true;
	
	if(!userModel)
		return true;
	
	QStringList removeConstraints;
	removeConstraints << QString("materialsId=%1").arg(medium->id());
	for(int i = 0; i < ownerModel->rowCount(); i++){
		Person *person = ownerModel->personFromIndex(ownerModel->index(i,0));
		if(!person)
			continue;
		removeConstraints << QString("usersId<>%1").arg(person->id());
	}
	
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	bool transactions = false;
	if(!useOldTransaction){
		transactions =  db.driver()->hasFeature(QSqlDriver::Transactions);
		if(transactions)
			db.transaction();
	}
	
	// remove not needed users
	QSqlQuery query;
	if(!query.exec(QString("DELETE FROM userownmaterials WHERE %1").arg(removeConstraints.join(" AND ")))){
		if(transactions)
			db.rollback();
		if(closeDB)
			db.close();
		return false;
	}
	
	// add new users
	QList<int> presentIds;
	if(!query.exec(QString("SELECT * FROM userownmaterials WHERE materialsId=%1").arg(medium->id()))){
		if(transactions)
			db.rollback();
		if(closeDB)
			db.close();
		return false;
	}
	while(query.next()){
		presentIds << query.record().value("usersId").toUInt();
	}
	
	for(int i = 0; i < ownerModel->rowCount(); i++){
		Person *person = ownerModel->personFromIndex(ownerModel->index(i,0));
		if(!person)
			continue;
		if(presentIds.indexOf(person->id()) == -1){
			query.prepare("INSERT INTO userownmaterials (materialsId, usersId, boughtPriceCurrenciesId)"
										"  VALUES(:materialsId, :usersId, :boughtPriceCurrenciesId)");
			query.bindValue(":materialsId", medium->id());
			query.bindValue(":usersId", person->id());
			query.bindValue(":boughtPriceCurrenciesId", medium->boughtPriceCurrency());
			if(!query.exec()){
				if(transactions)
					db.rollback();
				if(closeDB)
					db.close();
				return false;
			}
		}
	}
	if(transactions)
		db.commit();
	if(closeDB)
		db.close();
	return true;
}

void MediumOwnerWidget::ownerSelected ( QModelIndex index )
{
	ownerIndex = index;
	
	removeButton->setEnabled( false );
	if(ownerIndex.isValid())
		removeButton->setEnabled(true);
}

void MediumOwnerWidget::ownerDoubleClicked(QModelIndex index)
{
	ownerSelected(index);
	removeOwner();
}

void MediumOwnerWidget::userSelected ( QModelIndex index )
{
	userIndex = index;
	
	addButton->setEnabled( false );
	if(userIndex.isValid())
		addButton->setEnabled(true);
}

void MediumOwnerWidget::userDoubleClicked(QModelIndex index)
{
	userSelected(index);
	addOwner();
}

void MediumOwnerWidget::addOwner()
{
	QModelIndexList selected = userView->selectionModel()->selectedRows();
	foreach(QModelIndex index, selected){
		Person *person = userModel->takePerson(index);
		if(!person)
			return;
	
		ownerModel->addPerson(person);
		userSelected(QModelIndex());
		ownerSelected(QModelIndex());
	}
}

void MediumOwnerWidget::removeOwner()
{
	QModelIndexList selected = ownerView->selectionModel()->selectedRows();
	foreach(QModelIndex index, selected){
		Person *person = ownerModel->takePerson(index);
		if(!person)
			return;
		
		userModel->addPerson(person);
		userSelected(QModelIndex());
		ownerSelected(QModelIndex());
	}
}

void MediumOwnerWidget::init()
{
	medium = 0;
	ownerModel = 0;
	userModel = 0;
	
	setupUi(this);
	connect(addButton, SIGNAL(clicked()), this, SLOT(addOwner()));
	connect(removeButton, SIGNAL(clicked()), this, SLOT(removeOwner()));
	connect(ownerView, SIGNAL(clicked(QModelIndex)), this, SLOT(ownerSelected(QModelIndex)));
	connect(ownerView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(ownerDoubleClicked(QModelIndex)));
	connect(userView, SIGNAL(clicked(QModelIndex)), this, SLOT(userSelected(QModelIndex)));
	connect(userView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(userDoubleClicked(QModelIndex)));
	
	ownerSelected(QModelIndex());
	userSelected(QModelIndex());
	
#if QT_VERSION_MINOR < 6
        addButton->setIcon(QIcon(":/icons/1left.png"));
        removeButton->setIcon(QIcon(":/icons/1right.png"));
#else
        addButton->setIcon(QIcon::fromTheme("arrow-left", QIcon(":/icons/1left.png")));
        removeButton->setIcon(QIcon::fromTheme("arrow-right", QIcon(":/icons/1right.png")));
#endif
}

#include "mediumownerwidget.moc"
