/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MIMETYPE_H
#define MIMETYPE_H

#include "libmacro.h"

#include "mimemodelitem.h"

class Program;

#include <QString>
#include <QList>
#include <QFileInfo>

class EXPORT_DLL MimeType : public MimeModelItem
{
  public:
    MimeType ( QString newName = "" );
    ~MimeType();
    void setName ( QString newName );
		/**
			* mime type name is for example application/pdf
			*/
    QString name() const;
    /**
      * suffix means *.suffix like bla/bla/bla/bli.pdf the suffix is pdf
			* suffix and prefix can not be set the same time, so this clears prefix
      */
    void setSuffix ( QString suffix );
    QString suffix() const;
    /**
      * prefix means something like protocol
			* clears suffix (@see setSuffix())
      */
    void setPrefix ( QString prefix );
    QString prefix() const;

    /**
      * sets the default application for this mimetype
			* @todo possiblePrograms().contains(Program*) isn't working!
      */
    void setDefaultProgram ( Program *newProgram );
		void setDefaultProgram ( QString newProgram );
    Program *defaultProgram() const;
    void setPossibleProgram ( QString possibleProgram );
    void setPossibleProgram ( QFileInfo possibleProgram );
    void setPossibleProgram ( Program *possibleProgram );
    QList<Program*> possiblePrograms() const;

    int rowCount() const;

    bool saveMimeType();
    bool loadMimeType ( QString newName = "" );

    bool operator== ( const MimeType &compareType ) const;
    bool operator!= ( const MimeType &compareType ) const;

  private:
    QString myName;
    QString mySuffix, myPrefix;
    Program *myDefaultProgram;
    QList<Program*> myPossiblePrograms;
};

#endif // MIMETYPE_H
