/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MINVALIDATOR_H
#define MINVALIDATOR_H

#include "libmacro.h"

#include <QValidator>
#include <QStringList>

class MINValidatorPlugin;
class ModuleListWidgetItem;

/**
	* This class does all stuff around validate, check and convert MIN's
	* @note not really usefull for QLineEdit's right now
	* @todo unloading plugins isn't working
	*/
class EXPORT_DLL MINValidator : public QValidator
{
	Q_OBJECT
	public:
		MINValidator(QObject *parent = 0);
    ~MINValidator();
		QList<ModuleListWidgetItem*> loadProtocol();
		/**
			* @return a list of all known types
			*/
		QStringList supportedTypes() const;
		QString descriptionToType(QString type) const;
		bool isSupported(QString type) const;

		void fixup ( QString &input ) const;
		QValidator::State validate ( QString &input, int &pos ) const;
		
	public slots:
		/**
			* @return the type of @p min, if not known, empty QString is returned
			*/
		QString type(QString min) const;
		/**
			* @return a list of types, which might be valid
			*/
		QStringList possibleTypes(QString min) const;
		/**
			* tries to check wether @p min is valid or not
			* @return 0 - if @p min is not valid or no plugin was found
			* @return 1 - if @p min is valid
			*/
		bool isValid(QString min, QString type = QString()) const;
		/**
			* tries to generate a valid number of type @p type out of @p min
			* @return resulting min (which might a totally different number) or empty, if something went really really wrong
			* @attention This function should be used carefully, because with really corrupted numbers as @p min,
			* @attention the result may be a totally different valid number
			*/
		QString validate(QString min, QString type) const;
		/**
			* tries to convert @p min from @p fromType to @p toType
			* @return the converted MIN is returned, if conversion was not successfull, empty QString is returned
			*/
		QString convert(QString min, QString toType, QString fromType = QString()) const;
		/**
			* strippes out all not needed stuff
			*/
		QString strip(QString min, QString type = QString()) const;
		
	private:		
		void unloadPlugins();
		
		QList<MINValidatorPlugin*> plugins;
		QList<ModuleListWidgetItem*> pluginProtocol;
		QStringList types;
};

#endif // MINVALIDATOR_H
