/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MINVALIDATORPLUGIN_H
#define MINVALIDATORPLUGIN_H

#include "libmacro.h"

#include <QObject>

class EXPORT_DLL MINValidatorPlugin : public QObject
{
  public:
    virtual ~MINValidatorPlugin() {}

		/**
			* @return the name of the MIN (e.g. EAN, ISBN, ISSN)
			* @return should be not translated
			*/
    virtual QString type() const = 0;
		/**
			* @return a description, to summarize the use cases and specialities of this type
			* @return should be translated
			*/
		virtual QString description() const = 0;
		/**
			* @return true if this plugin can convert a min of own type to @p toType
			* E.g.: ISBN validator should be able to convert a ISBN-10 into an EAN
			*/
		virtual bool canConvert(QString toType) const = 0;
		/**
			* do the conversion, but leave out the checksum
			* @return result without checksum or empty string, if convert failes
			*/
		virtual QString convert(QString min, QString toType) const = 0;
		/**
			* check wether @p min is valid as is
			*/
		virtual bool isValid(QString min) const = 0;
		/**
			* made @p min a valid min of own type (strings - and blanks are removed)
			* @param min with or without a checksum
			* @return the min with valid checksum append or empty string if calculation failed
			* @return @p min is returned untouched, if it is already valid
			* @attention This function should be used carefully, because with really corrupted numbers as @p min,
			* @attention the result may be a different valid number
			*/
		virtual QString validate(QString min) const = 0;
		/**
			* @return true if min is of own type
			*/
		virtual bool isType(QString min) const = 0;
				/**
			* @return true, if @p min might be a MIN of own type in future (see QValidator state intermediate)
			* @return or if all not allowed strings are removed
			*/
		virtual bool mayType(QString min) const = 0;
		/**
			* strip out all not needed stuff out of min
			* @return pure needed characters, validity checks should not be done
			*/
		virtual QString strip(QString min) const = 0;
		
		virtual bool operator== (const MINValidatorPlugin &plugin) const = 0;
    virtual bool operator!= (const MINValidatorPlugin &plugin) const = 0;
};

Q_DECLARE_INTERFACE ( MINValidatorPlugin,
                      "Meganizer.MINValidatorPlugin/0.0.2" );

#endif // MINVALIDATORPLUGIN_H
