/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MPUSHBUTTON_H
#define MPUSHBUTTON_H

#include "libmacro.h"

#include <QPushButton>

/**
	* This button hides the text, if:
	*  - iconAndText is false
	*  - and icon is not null
	*  - and showText is false (Default)
	*/
class EXPORT_DLL MPushButton : public QPushButton
{
	Q_OBJECT
	/**
		* if set to true, the text will be shown in any case
		* if set to false (default), the text will only be shown, if no icon is set
		* or iconAndText is true
		*/
	Q_PROPERTY(bool showText READ showText WRITE setShowText)
	public:
		MPushButton ( QWidget *parent = 0 );
		MPushButton ( const QString & text, QWidget * parent = 0 );
		MPushButton ( const QIcon & icon, const QString & text, QWidget * parent = 0 );
		void setShowText(bool anytime = true);
		bool showText() const;
		void setIcon ( const QIcon & icon );
		/**
			* tries to use QIcon::fromTheme if possible, otherwise QIcon(@p iconPath) is used (Requires Qt 4.6).
			* This is also the case, if the standard theme icon is not present
			*/
		void setIcon ( QString iconPath, QString standardIconName);
		
	private:
		void init();
		
		bool myShowText;
		QString myText;
};

#endif // MPUSHBUTTON_H
