/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include  "mwidget.h"

#include <QSettings>
#include <QApplication>

MWidget::MWidget ( QWidget *parent, Qt::WindowFlags flags ) : QWidget ( parent, flags )
{
  QSettings qSettings;
  iconsAndText = qSettings.value ( "iconsAndText" ).toBool();
	alternatingRows = qSettings.value("alternatingRowColors").toBool();
}

MWidget::~MWidget()
{

}

void MWidget::connectByPass ( QObject *object )
{
  connect ( object, SIGNAL ( error ( const QString& ) ), this, SLOT ( errorByPass ( const QString& ) ) );
  connect ( object, SIGNAL ( warning ( const QString& ) ), this, SLOT ( warningByPass ( const QString& ) ) );
  connect ( object, SIGNAL ( information ( const QString& ) ), this, SLOT ( informationByPass ( const QString& ) ) );

  connect ( object, SIGNAL ( addMenu ( QMenu*, const QString& ) ), this, SLOT ( addMenuByPass ( QMenu*, const QString& ) ) );
  connect ( object, SIGNAL ( delMenu ( QMenu* ) ), this, SLOT ( delMenuByPass ( QMenu* ) ) );
  connect ( object, SIGNAL ( addAction ( QAction *, const QString& ) ), this, SLOT ( addActionByPass ( QAction*, const QString& ) ) );
  connect ( object, SIGNAL ( delAction ( QAction* ) ), this, SLOT ( delActionByPass ( QAction* ) ) );
  connect ( object, SIGNAL ( addToolbar ( QToolBar * ) ), this, SLOT ( addToolbarByPass ( QToolBar* ) ) );
  connect ( object, SIGNAL ( delToolbar ( QToolBar* ) ), this, SLOT ( delToolbarByPass ( QToolBar* ) ) );

  connect ( object, SIGNAL ( addNewDockWidget ( QDockWidget*, const Qt::DockWidgetArea& ) ), this, SLOT ( addNewDockWidgetByPass ( QDockWidget*, const Qt::DockWidgetArea& ) ) );
  connect ( object, SIGNAL ( removeDockWidget ( QDockWidget* ) ), this, SLOT ( removeDockWidgetByPass ( QDockWidget* ) ) );
  connect ( object, SIGNAL ( setCentralWidget ( QWidget* ) ), this, SLOT ( setCentralWidgetByPass ( QWidget* ) ) );
  connect ( object, SIGNAL ( showSettings() ), this, SLOT ( showSettingsByPass() ) );
  connect ( object, SIGNAL ( showHelp( QString ) ), this, SLOT ( showHelpByPass( QString ) ) );
  connect ( object, SIGNAL ( showStatusWidget ( QWidget* ) ), this, SLOT ( showStatusWidgetByPass ( QWidget* ) ) );
  connect ( object, SIGNAL ( removeStatusWidget ( QWidget* ) ), this, SLOT ( removeStatusWidgetByPass ( QWidget* ) ) );

//   connect(this, SIGNAL(reloadSettings()), object, SLOT(reloadSettingsByPass()));
}

void MWidget::setWindowTitle ( const QString &title )
{
  QWidget::setWindowTitle ( QString("%1 - %2").arg(QApplication::applicationName()).arg(title) );
}

void MWidget::settingsChanged() {}

void MWidget::print() {}

void MWidget::save() {}

void MWidget::doExport() {}

void MWidget::redo() {}

void MWidget::undo() {}

void MWidget::cut() {}

void MWidget::copy() {}

void MWidget::paste() {}

void MWidget::reload() {}

void MWidget::errorByPass ( const QString &string )
{
  emit error ( string );
}

void MWidget::warningByPass ( const QString &string )
{
  emit warning ( string );
}

void MWidget::informationByPass ( const QString &string )
{
  emit information ( string );
}

void MWidget::addMenuByPass ( QMenu *newMenu, const QString &parent )
{
  emit addMenu ( newMenu, parent );
}

void MWidget::delMenuByPass ( QMenu *del )
{
  emit delMenu ( del );
}

void MWidget::addActionByPass ( QAction *newAction, const QString &parent )
{
  emit addAction ( newAction, parent );
}

void MWidget::delActionByPass ( QAction *del )
{
  emit delAction ( del );
}

void MWidget::addToolbarByPass ( QToolBar *newBar )
{
  emit addToolbar ( newBar );
}

void MWidget::delToolbarByPass ( QToolBar *delBar )
{
  emit delToolbar ( delBar );
}

void MWidget::addNewDockWidgetByPass ( QDockWidget *widget, const Qt::DockWidgetArea &area )
{
  emit addNewDockWidget ( widget, area );
}

void MWidget::removeDockWidgetByPass ( QDockWidget *dockWidget )
{
  emit removeDockWidget ( dockWidget );
}

void MWidget::setCentralWidgetByPass ( QWidget *centralWidget )
{
  emit setCentralWidget ( centralWidget );
}

void MWidget::dbLoginByPass()
{
  emit dbLogin();
}

void MWidget::showSettingsByPass()
{
  emit showSettings();
}

void MWidget::reloadSettingsByPass()
{
  emit reloadSettings();
}

void MWidget::showHelpByPass( QString file)
{
  emit showHelp( file );
}

void MWidget::showStatusWidgetByPass ( QWidget *widget )
{
  emit showStatusWidget ( widget );
}

void MWidget::removeStatusWidgetByPass ( QWidget *widget )
{
  emit removeStatusWidget ( widget );
}

#include "mwidget.moc"
