/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "openfileexternal.h"

#include "mimetypemodel.h"
#include "searchexternalapp.h"
#include "mimetype.h"
#include "program.h"

#include <QFileInfo>
#include <QStringList>
#include <QSettings>
#include <QProcess>

#include <QDebug>

OpenFileExternal::OpenFileExternal(QObject *parent) : MObject(parent){
  model = 0;
	myType = 0;
  model = mimeTypes();
  isStream = false;
}

OpenFileExternal::~OpenFileExternal(){
	delete model;
}

MimeTypeModel* OpenFileExternal::mimeTypes(){
  MimeTypeModel *myModel = new MimeTypeModel();
  myModel->loadModel();
  return myModel;
}

void OpenFileExternal::setFile(QString file, QString mimeType){
  myFile = file;
  myType = model->getMimeType(mimeType);
  
  if(myFile.contains("://")){
    isStream = true;
  }
  else{
    myFile = QFileInfo(myFile).absoluteFilePath();
  }
}

void OpenFileExternal::setFile(QFileInfo file, QString mimeType){
  setFile(file.absoluteFilePath(), mimeType);
}

bool OpenFileExternal::openFile(QString file, QString mimeType){
  setFile(file, mimeType);
  return open();
}

bool OpenFileExternal::openFile(QFileInfo file, QString mimeType){
  setFile(file, mimeType);
  return open();
}

bool OpenFileExternal::open(){
  if(!isStream){
    if(!QFileInfo(myFile).isReadable())
      return false;
  }

  // detect mime type
  QFileInfo app;
  if(!model)
    return false;
  MimeType *mimeType = detectType();
  if(mimeType)
    if(mimeType->defaultProgram())
      app = mimeType->defaultProgram()->file();

  QSettings qSettings;
  qSettings.beginGroup("OpenFileExternal");
  bool showAnytime = qSettings.value("showAnytime", false).toBool();
  qSettings.endGroup();
    
  // try to open file into app
  // first check wether app is executable
  if((!app.isExecutable()) || (showAnytime)){
    app.setFile(searchApp(mimeType));
    if(!app.isExecutable()){
      emit information(tr("Don't have an executeable application: %1").arg(app.absoluteFilePath()));
      return false;
    }
  }
  // it's executable, so start it
  QStringList arguments;
  arguments.append(myFile);
  emit information(tr("Starting %1 %2").arg(app.absoluteFilePath()).arg(arguments.join(", ")));
  return QProcess::startDetached(app.absoluteFilePath(), arguments);
}

MimeType *OpenFileExternal::detectType(){
  if(!model)
    return false;
	
	if(myType)
		return myType;
  
  if(isStream){
		// test prefix
		//   QString prefix = myFile.suffix().toLower();
		for(int i = 0; i < model->rowCount(); i++){
			QModelIndex index = model->index(i, 2);
			QString prefix = model->data(index, Qt::DisplayRole).toString();
			if(prefix.isEmpty())
				continue;
			QRegExp reg("^("+ prefix +")");
			if(reg.indexIn(myFile) >= 0)
				return model->getMimeType(i);
		}
		return 0;
	}
  
  // test suffix
	QString suffix = QFileInfo(myFile).suffix().toLower();
	for(int i = 0; i < model->rowCount(); i++){
		QModelIndex index = model->index(i, 1);
		if(model->data(index, Qt::DisplayRole).toString() == suffix){
			return model->getMimeType(i);
		}
	}
	return 0;
}

QString OpenFileExternal::searchApp(MimeType* mimeType){
  if(!model)
    return false;
  SearchExternalApp *searchApp;
  searchApp = new SearchExternalApp();
  QList<Program*> progList;
  if(mimeType){
    progList = mimeType->possiblePrograms();
  }
  connectByPass(searchApp);
  searchApp->init(progList);
  if(searchApp->exec() != QDialog::Accepted){
		delete searchApp;
    return QString();
	}

  QString result = searchApp->selectedApp();
	bool setAsDefault = searchApp->setAsDefault();
	delete searchApp;
  if(!QFileInfo(result).isExecutable())
    return QString();

  if(setAsDefault){
		if(mimeType){
      mimeType->setDefaultProgram(new Program(result, mimeType));
      mimeType->saveMimeType();
    }
  }

  return result;
}

#include "openfileexternal.moc"
