/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "person.h"

#include "msqldatabase.h"

#include <QSqlQuery>
#include <QSqlRecord>
#include <QVariant>

Person::Person ( QObject* parent )
	: QObject ( parent )
{
	init();
}

Person::Person ( unsigned int personId, QObject* parent )
	: QObject ( parent )
{
	init(personId);
}

Person::Person ( QString newName, QString newGivenName, QObject* parent )
	: QObject ( parent )
{
	init();
	setName(newName, newGivenName);
}

Person::~Person()
{

}

bool Person::isValid()
{
	if(name().isEmpty())
		return false;
	return true;
}

void Person::setId ( unsigned int personId, bool setPName )
{
	myPersonId = personId;
	if(setPName){
		setName("", false);
		setGivenName("");
		if(personId > 0){
			MSqlDatabase db = MSqlDatabase::database();
			bool closeDB = !db.isOpen();
			if(closeDB)
				db.open();
			if(db.isOpen()){
				QSqlQuery query;
				query.exec(QString("SELECT * FROM materialpersons WHERE id=%1").arg(personId));
				if(query.first()){
					setName(query.record().value("name").toString(), false);
					setGivenName(query.record().value("givenName").toString());
				}
				if(closeDB)
					db.close();
			}
		}
	}
  emit changed(this);
}

unsigned int Person::id() const
{
	return myPersonId;
}

void Person::setName ( QString newName, QString givenName, bool setPId )
{
	setGivenName(givenName);
	setName(newName, setPId);
}

void Person::setName ( QString newName, bool setPId )
{
	myName = newName.trimmed();
	if(setPId){
		setId(0, false);
		if(!myName.isEmpty()){
			MSqlDatabase db = MSqlDatabase::database();
			bool closeDB = !db.isOpen();
			if(closeDB)
				db.open();
			if(db.isOpen()){
				QSqlQuery query;
				query.prepare("SELECT * FROM materialpersons WHERE name=:name, givenName=:givenName");
				query.bindValue(":name", newName);
				query.bindValue(":givenName", givenName());
				query.exec();
				if(query.size() == 1){
					if(query.first()){
						setId(query.record().value("id").toUInt(), false);
						setGivenName(query.record().value("givenName").toString());
					}
				}  
				if(closeDB)
					db.close();
			}
		}
	}
  emit changed(this);
}

QString Person::name() const
{
	return myName;
}

void Person::setGivenName ( QString newName )
{
	myGivenName = newName.trimmed();
  emit changed(this);
}


QString Person::givenName() const
{
	return myGivenName;
}

QString Person::combine() const
{
	if(givenName().isEmpty())
		return name();
	
	return QString("%1, %2").arg(name()).arg(givenName().left(1));
}

void Person::setMarkForRemovel( bool remove )
{
	shouldBeRemoved = remove;
	emit changed(this);
}

bool Person::markForRemovel() const
{
	return shouldBeRemoved;
}

Person* Person::copyPerson()
{
	Person *newP = new Person();
	newP->setId(id(), false);
	newP->setName(name(), false);
	newP->setGivenName(givenName());
	if(markForRemovel())
		newP->setMarkForRemovel();
	return newP;
}

bool Person::operator== ( const Person& person ) const
{
	if((id() > 0) || (person.id() > 0)){
		if(id() == person.id())
			return true;
		return false;
	}
	
	if(name() != person.name())
		return false;
	
	if(givenName() != person.givenName())
		return false;
	
	return true;
}

bool Person::operator!= ( const Person& person ) const
{
	return !operator==(person);
}

void Person::init(int personId)
{
  myPersonId = personId;
  shouldBeRemoved = false;
}

#include "person.moc"
