/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef QUERYTHREAD_H
#define QUERYTHREAD_H

#include "libmacro.h"
#include <QThread>

class QueryWorker;

#include <QSqlQuery>
#include <QSqlRecord>

/**
	* Can be used to execute asynchronous SQL-Queries.
	* Should be stopped right that:
	* @see QueryThread.cancelQuery() to stop running query
	* @see QThread::quit() to stop eventloop
	* @see QThread::wait(1000) to give the thread max 1s to be stopped
	*/
class EXPORT_DLL QueryThread : public QThread
{
	Q_OBJECT
  public:
    QueryThread( QObject* parent = 0 );
    void run();
		bool isStarted();
		bool waitForStart();
		
	public slots:
		/**
			* prepares an QSqlQuery with query @p queryString.
			* Afterwards it binds all values from @p bindValues. This values
			* must be ordered (Placeholder ? not :fieldName)
			*/
		void execQuery(QString queryString, QList<QVariant> bindValues);
		void cancelQuery();
		void dbSetup(QString dbname, QString username, QString password, QString host, QString driver);
		
	private slots:
		void resultByPass(QSqlRecord record);
		void setRangeByPass(int min, int max);
		void setValueByPass(int value);
		void finishedByPass(bool status);
		void setStarted();
		
	private:
		/**
			* this object run's completely at the other thread
			* therefore the communication is done through the event loop 
			* with help of signals and slots
			*/
		QueryWorker *worker;
		bool finishedStartup;
		
	signals:
		/**
			* used to configure database at QueryWorker
			*/
		void configureDB(QString dbname, QString username, QString password, QString host, QString driver);
		/**
			* used to get the query to the QueryWorker instance
			*/
		void addToQueue(QString queryString, QList<QVariant> bindValues);
		void cancel();
		/**
			* used to populate result back to caller
			*/
		void result(QSqlRecord record);
		void setRange(int min, int max);
		void setValue(int value);
		void finished(bool status);
};

#endif // QUERYTHREAD_H
