/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "queryworker.h"

#include <QSqlError>
#include <QDateTime>

QueryWorker::QueryWorker(QObject* parent): MObject(parent)
{
}

QueryWorker::~QueryWorker()
{
	MSqlDatabase::removeDatabase(connectionName);
}

void QueryWorker::configureDB(QString dbname, QString username, QString password, QString host, QString driver)
{
	// make sure that really every thread has it's own db name
	connectionName = "QueryWorker-"+ QDateTime::currentDateTime().toString("yyyyMMddhhmmzzz");
	MSqlDatabase db = MSqlDatabase::addDatabase(driver, connectionName);
	db.setDatabaseName(dbname);
	db.setUserName(username);
	db.setPassword(password);
	db.setHostName(host);
}

void QueryWorker::addToQueue(QString queryString, QList<QVariant> bindValues)
{
	MSqlDatabase db = MSqlDatabase::database(connectionName);
	abort = false;
	if(!db.open()){
		emit finished(false);
		db.close();
		return;
	}
	
	if(abort){
		emit finished(false);
		db.close();
		return;
	}
	
	// db is needed here, because otherwise the default connection would be used!
	QSqlQuery query(db);
	query.prepare(queryString);
	foreach(QVariant bind, bindValues)
		query.addBindValue(bind);
	
	if(!query.exec()){
		emit finished(false);
		db.close();
		return;
	}
	
	if(abort){
		emit finished(false);
		db.close();
		return;
	}
	
	int max = query.size();
	if(max < 1){
		emit finished(false);
		db.close();
		return;
	}
	
	emit setRange(0, max);
	
	int i = 0;
	while(query.next()){
		if(abort){
			emit finished(false);
			db.close();
			return;
		}
		emit result(query.record());
		i++;
		emit setValue(i);
	}
	
	db.close();
	emit finished(true);
}

void QueryWorker::cancel()
{
	abort = true;
}

#include "queryworker.moc"
