/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "ui_searchexternalapp.h"
#include "searchexternalapp.h"

#include "program.h"

#include <QListWidgetItem>
#include <QFileDialog>
#include <QFileInfo>
#include <QFileIconProvider>
#include <QSettings>

SearchExternalApp::SearchExternalApp ( QWidget *parent ) : MDialog ( parent ), m_ui ( new Ui::SearchExternalApp )
{
  m_ui->setupUi ( this );

  connect ( m_ui->searchAppButton, SIGNAL ( clicked() ), this, SLOT ( searchApp() ) );
  connect ( m_ui->cancelButton, SIGNAL ( clicked() ), this, SLOT ( reject() ) );
  connect ( m_ui->acceptButton, SIGNAL ( clicked() ), this, SLOT ( accept() ) );
  connect ( m_ui->foundAppsView, SIGNAL ( currentTextChanged ( QString ) ), this, SLOT ( itemChanged ( QString ) ) );

	setHelp("keyword:External MimeTypes", m_ui->helpButton);
}

SearchExternalApp::~SearchExternalApp()
{
  this->close();
  delete m_ui;
}

void SearchExternalApp::init ( QList<Program *>findAppsList )
{
  apps = findApps ( findAppsList );
  QFileIconProvider iconProvider;
  foreach ( Program* app, apps ) {
    QListWidgetItem *item = new QListWidgetItem ( iconProvider.icon ( app->file() ), app->file().baseName(), m_ui->foundAppsView );
    m_ui->foundAppsView->addItem ( item );
  }
}

QString SearchExternalApp::selectedApp()
{
  if ( ( curApp.isEmpty() ) && ( apps.size() > 0 ) )
    return apps[0]->file().absoluteFilePath();

  return curApp;
}

bool SearchExternalApp::setAsDefault()
{
  if ( m_ui->defaultEdit->isChecked() )
    return true;

  return false;
}

QList<QDir> SearchExternalApp::defaultPaths()
{
  QList<QDir> paths;
#if defined(Q_OS_WIN32)
  paths.append ( QDir ( tr ( "c:/Program Files" ) ) );
  paths.append ( QDir ( tr ( "c:/Program Files (x86)" ) ) );
#else
  paths.append ( QDir ( "/usr/bin" ) );
  paths.append ( QDir ( "/bin" ) );
  paths.append ( QDir ( "/usr/local/bin" ) );
  paths.append ( QDir ( "/opt/bin" ) );
#endif
  return paths;
}

int SearchExternalApp::exec()
{
  QSettings qSettings;
  qSettings.beginGroup ( "OpenFileExternal" );
  bool showAnytime = qSettings.value ( "showAnytime", false ).toBool();
  qSettings.endGroup();

  if ( ( !showAnytime ) && ( apps.size() == 1 ) ) {
    curApp = apps[0]->file().absoluteFilePath();
    return QDialog::Accepted;
  }

  return QDialog::exec();
}

void SearchExternalApp::searchApp()
{
  QString dir;
  QList<QDir> dirs = defaultPaths();

  if ( dirs.size() > 0 )
    dir = dirs[0].absolutePath();

  QString app = QFileDialog::getOpenFileName ( this, tr ( "Search Applications" ), dir );

  if ( app.isEmpty() )
    return;

  Program *application = new Program ( app );

  apps.append ( application );

  QListWidgetItem *item = new QListWidgetItem ( QIcon(), application->file().baseName() );

  m_ui->foundAppsView->addItem ( item );

  m_ui->foundAppsView->setCurrentItem ( item );
}

void SearchExternalApp::itemChanged ( QString currentText )
{
  foreach ( Program *app, apps ) {
    if ( app->file().baseName() == currentText ) {
      curApp = app->file().absoluteFilePath();
      return;
    }
  }
}

QList<Program*> SearchExternalApp::findApps ( QList<Program*> appNames )
{
  QList<QDir> paths = defaultPaths();

  QList<Program*> results;
  foreach ( Program* app, appNames ) {
    foreach ( QDir path, paths ) {
      // extend the searchpaths with the path of the application
      path.setPath ( path.path() +"/"+ app->file().path() );
      QFileInfo info;
      info.setFile ( path , app->file().fileName() );

      if ( info.isExecutable() ) {
        results.append ( new Program ( info.absoluteFilePath() ) );
      }
    }
  }

  return results;
}

#include "searchexternalapp.moc"
