/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SEARCHPLUGIN_H
#define SEARCHPLUGIN_H

#include "libmacro.h"

#include "buydata.h"
#include "sqlconstraint.h"

#include <QObject>
#include <QMap>

class QIcon;
class QMenu;
class QAction;
class QToolBar;
class QDockWidget;
class QAbstractItemModel;
class QStringList;

class SettingsWidget;
class SetupWizardObject;
class MWidget;
class MediaModel;
class Medium;
class SearchPlugin;

class EXPORT_DLL SearchPlugin : public QObject
{
  public:
    virtual ~SearchPlugin() {}

    virtual bool initiate() = 0;
    virtual bool unload() = 0;
    virtual QString name() const = 0;
    virtual QString settingsGroup() = 0;
    virtual QIcon icon() = 0;
    virtual QString description() = 0;
		/**
			* a list of supported attributes (key, translation)
			* This list should contain only special attributes, not supported by Medium.
			*/
    virtual QMap<QString, QString> supportedAttributes() = 0;
    /**
    	* A list of supported Categories which can be searched (key, translation).
    	* The categories should represent different material types.
    	* @see Amazon
    	*/
    virtual QMap<QString, QString> supportedCategories() = 0;
    virtual SettingsWidget *settingsWidget() = 0;
    virtual SetupWizardObject *wizardObject() = 0;
		/**
			* If any medium is created, the source should be setted to Medium::SearchPlugin using
			* setSource(Medium::SearchPlugin)
			*/
    virtual bool search ( MediaModel *resultModel, QList<SqlConstraint> sqlConstraints ) = 0;
    virtual void recieveBuyData ( Medium *medium ) = 0;
    virtual bool purchase ( QList<BuyData> items, QMap<Medium*, int> quantities ) = 0;
    virtual bool operator== (const SearchPlugin &plugin) const = 0;
    virtual bool operator!= (const SearchPlugin &plugin) const = 0;

  public slots:
    virtual void show() = 0;

    // MObject slots
    virtual void settingsChanged() = 0;
    virtual void print() = 0;
    virtual void save() = 0;
    virtual void doExport() = 0;
    virtual void redo() = 0;
    virtual void undo() = 0;
    virtual void cut() = 0;
    virtual void copy() = 0;
    virtual void paste() = 0;
    virtual void reload() = 0;
    virtual void reloadSettingsByPass() = 0;

  signals:
    virtual void foundBuyData ( const BuyData &data ) = 0;

    virtual void error ( const QString &string ) = 0;
    virtual void warning ( const QString &string ) = 0;
    virtual void information ( const QString &string ) = 0;

    virtual void addMenu ( QMenu* newMenu, const QString &parent ) = 0;
    virtual void delMenu ( QMenu *del ) = 0;
    virtual void addAction ( QAction* newMenu, const QString &parent ) = 0;
    virtual void delAction ( QAction *del ) = 0;
    virtual void addToolbar ( QToolBar* newbar ) = 0;
    virtual void delToolbar ( QToolBar* delBar ) = 0;

    virtual void addNewDockWidget ( QDockWidget *widget, const Qt::DockWidgetArea &area ) = 0;
    virtual void setCentralWidget ( QWidget *centralWidget ) = 0;
    virtual void removeDockWidget ( QDockWidget *widget ) = 0;

    virtual void dbLogin() = 0;

    virtual void showSettings() = 0;
    virtual void reloadSettings() = 0;

    virtual void showHelp( const QString &page ) = 0;

    virtual void showStatusWidget ( QWidget *widget ) = 0;
    virtual void removeStatusWidget ( QWidget *widget ) = 0;
};

Q_DECLARE_INTERFACE ( SearchPlugin,
                      "Meganizer.SearchPlugin/0.3.4" );

#endif // SEARCHPLUGIN_H
