/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "sqlfield.h"

SqlField::SqlField()
{
    init();
}

SqlField::SqlField ( QString myName, type myType, bool notNull, bool autoIncrement )
{
    init();
    fieldName = myName;
    dataType ( myType );
    nn = notNull;
    ai = autoIncrement;
}


void SqlField::init()
{
    fieldType = SqlField::noType;
    pk = false;
    ai = false;
    nn = false;
    zf = false;
    fieldUnsigned = false;
    binary = false;
    fieldLength = 0;

    convFieldTypes.insert ( SqlField::integer, "INT" );
    convFieldTypes.insert ( SqlField::boolean, "" );
    convFieldTypes.insert ( SqlField::text, "TEXT" );
    convFieldTypes.insert ( SqlField::varchar, "VARCHAR" );
    convFieldTypes.insert ( SqlField::date, "DATE" );
		convFieldTypes.insert ( SqlField::time, "TIME" );
    convFieldTypes.insert ( SqlField::tinyint, "TINYINT" );
    convFieldTypes.insert ( SqlField::Double, "DOUBLE" );
    convFieldTypes.insert ( SqlField::timestamp, "TIMESTAMP" );
    convFieldTypes.insert(SqlField::blob, "BLOB");
}

bool SqlField::isValid() const
{
    if ( ( fieldName.isEmpty() ) || ( fieldType == SqlField::noType ) ) {
        return false;
    }

    if ( fieldType == SqlField::varchar ) {
        if ( fieldLength < 1 )
            return false;
    }

    if ( ( nn ) && ( zf ) ) {
        // zerofill and not null can not be used at same time
        return false;
    }

    return true;
}

QString SqlField::name() const
{
    return fieldName;
}

void SqlField::name ( QString myName )
{
    fieldName = myName;
}

bool SqlField::isPrimaryKey() const
{
    return pk;
}

void SqlField::isPrimaryKey ( bool status )
{
    pk = status;
}

bool SqlField::autoIncrement() const
{
    return ai;
}

void SqlField::autoIncrement ( bool status )
{
    ai = status;
}

bool SqlField::notNull() const
{
    return nn;
}

void SqlField::notNull ( bool status )
{
    nn = status;
}

QVariant SqlField::defaultValue() const
{
	bool check;
	fieldDefValue.toInt(&check);
	if ( check )
			return fieldDefValue;
	if(dataType() == SqlField::timestamp)
		return fieldDefValue;

   return QVariant();
}

void SqlField::defaultValue ( QVariant value )
{
    fieldDefValue = value;
}

SqlField::type SqlField::dataType() const
{
    return fieldType;
}

void SqlField::dataType ( SqlField::type myType )
{
    if ( myType == SqlField::boolean ) {
        fieldType = SqlField::tinyint;
        fieldLength = 1;
        return;
    }

    fieldType = myType;
}

unsigned int SqlField::length() const
{
    return fieldLength;
}

void SqlField::length ( unsigned int len )
{
    fieldLength = len;
}

bool SqlField::isUnsigned() const
{
    return fieldUnsigned;
}

void SqlField::isUnsigned ( bool status )
{
    fieldUnsigned = status;
}

bool SqlField::isBinary() const
{
    return binary;
}

void SqlField::isBinary ( bool status )
{
    binary = status;
}

bool SqlField::zeroFill() const
{
    return zf;
}

void SqlField::zeroFill ( bool status )
{
    zf = status;
}

QString SqlField::sqlFieldType() const
{
    QString result;

    if ( fieldType == noType )
        return result;

    result = convFieldTypes.value ( fieldType );

    if ( fieldLength > 0 ) {
        result += "(" + QString::number ( fieldLength ) + ")";
    }

    return result;
}

SqlField* SqlField::copy() const
{
	SqlField *newField = new SqlField(name(), fieldType, notNull(), autoIncrement());
	newField->isPrimaryKey(isPrimaryKey());
	newField->defaultValue(defaultValue());
	newField->length(length());
	newField->isUnsigned(isUnsigned());
  newField->isBinary(isBinary());
	newField->zeroFill(zeroFill());
	
	return newField;
}

bool SqlField::operator==(const SqlField& field) const
{
	if(field.name() == name())
		return true;
	
	return false;
}

bool SqlField::operator!=(const SqlField& field) const
{
	return !operator==(field);
}
