/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "sqltable.h"

SqlTable::SqlTable ( QString myName )
{
	idFieldName = "id";
  tableName = myName;
  fkActionMap.insert ( SqlTable::NoAction, "NO ACTION" );
  fkActionMap.insert ( SqlTable::SetNull, "SET NULL" );
  fkActionMap.insert ( SqlTable::Cascade, "CASCADE" );
  fkActionMap.insert ( SqlTable::Restrict, "RESTRICT" );
}

bool SqlTable::isValid()
{
  if ( tableName.isEmpty() ) {
    return false;
  }

  return true;
}

QString SqlTable::name()
{
  // seems to be, that the driver makes all names lower names
  return tableName.toLower();
}

void SqlTable::name ( QString myName )
{
  tableName = myName;
}

QList<SqlField*> SqlTable::fields()
{
  return myFields;
}

void SqlTable::appendField ( SqlField *append )
{
  myFields.append ( append );
}

SqlField * SqlTable::appendField ( QString myName, SqlField::type myType, bool notNull, bool autoIncrement )
{
  SqlField *result = new SqlField ( myName, myType, notNull, autoIncrement );
  appendField ( result );
  return result;
}

void SqlTable::appendFieldId ( )
{
	// id standard field
  // notNull and autoIncrement are set to true via constructor call
  // unsigned and primary key is seted manually below
	SqlField *fieldId = appendField ( idFieldName, SqlField::integer, true, true );
  fieldId->isPrimaryKey ( true );
  fieldId->isUnsigned ( true );
}

bool SqlTable::appendValues ( QStringList value )
{
  if ( value.size() != myFields.size() )
    return false;

  myValues.append ( value );

  return true;
}

QList<QStringList> SqlTable::values()
{
  return myValues;
}

void SqlTable::addForeignKey ( QString fieldName,
                               QString refTable,
                               QString refFieldName,
                               SqlTable::fkAction del,
                               SqlTable::fkAction update )
{
  QStringList tmp;
  tmp.append ( fieldName );
  tmp.append ( refTable );
  tmp.append ( refFieldName );
  tmp.append ( fkActionMap.value ( del ) );
  tmp.append ( fkActionMap.value ( update ) );
  constraints.append ( tmp );
}

void SqlTable::addForeignKey ( SqlField* ourField, SqlTable* refTable,
                         SqlTable::fkAction del, SqlTable::fkAction update )
{
	if(!ourField || !refTable)
		return;
	
	addForeignKey ( ourField->name(), refTable->name(), "id", del, update );
}

void SqlTable::addForeignKey ( SqlTable *refTable, QString newFieldName, 
															 SqlTable::fkAction del, SqlTable::fkAction update )
{
	if(!refTable)
		return;
	
	QString tablePart = refTable->name().right(refTable->name().length() - refTable->name().indexOf("_") - 1);
	
	// find id field
	QList<SqlField*> refFields = refTable->fields();
	foreach(SqlField *refField, refFields){
		if(refField->name() == idFieldName){
			if(newFieldName.isEmpty())
				newFieldName = tablePart + + "Id";
			SqlField *newField = refField->copy();
			newField->name(newFieldName);
			newField->isPrimaryKey(false);
			newField->autoIncrement(false);
			appendField(newField);
			addForeignKey(newField, refTable, del, update);
			return;
		}
	}
}

QString SqlTable::sqlConstraintsString ( SqlTable::databaseType type )
{
  QStringList results;
  foreach ( QStringList tmp, constraints ) {
    QString constraint;
    constraint  = " CONSTRAINT `";
    constraint += "fk_" + tableName + "_" + tmp[0] + "`\n";
    constraint += "  FOREIGN KEY (`" + tmp[0] + "`)\n";

    if ( type == SqlTable::SQLITE ) {
      constraint += "  REFERENCES " + tmp[1] + "(`" + tmp[2] + "`)\n";
    }

    else {
      constraint += "  REFERENCES `%db%`.`" + tmp[1] + "`(`" + tmp[2] + "`)\n";
    }

    constraint += "  ON DELETE " + tmp[3] + " ";

    constraint += "  ON UPDATE " + tmp[4];
    results.append ( constraint );
  }

  return results.join ( ",\n" );
}

void SqlTable::addIndex ( QString fieldName, unsigned int length, Qt::SortOrder order )
{
  // find field
	foreach(SqlField *field, fields()){
		if(field->name() == fieldName)
			return addIndex(field, length, order);
	}
}

void SqlTable::addIndex(SqlField* field, unsigned int length, Qt::SortOrder order)
{
	if(!field)
		return;
	
	QString orderString = "ASC";

	QString lengthString = "";
	if(length > 0){
		if(field->length() > 0){
			if(field->length() < length)
				length = field->length();
		}
		lengthString = QString("(%1)").arg(length);
	}
	
  if ( order == Qt::DescendingOrder )
    orderString = "DESC";

  myIndizes.append ( QString ( "INDEX `index_%1` (`%1`%2 %3)" )
                     .arg ( field->name() )
                     .arg ( lengthString )
                     .arg ( orderString )
                   );
}


QString SqlTable::indizes ()
{
  return myIndizes.join ( ",\n " );
}

