/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SQLTABLE_H
#define SQLTABLE_H

#include "libmacro.h"

#include "sqlfield.h"

#include <QString>
#include <QStringList>
#include <QMultiHash>

class EXPORT_DLL SqlTable
{
  public:
    enum fkAction { NoAction, SetNull, Cascade, Restrict };
    enum databaseType { Other, SQLITE };

    SqlTable ( QString myName = "" );
    bool isValid();
    QString name();
    void name ( QString myName );
    QList<SqlField*> fields();
    void appendField ( SqlField *append );
    SqlField *appendField ( QString myName = "", SqlField::type myType = SqlField::noType, bool notNull = false, bool autoIncrement = false );
		/**
			* append's the id standard field
			* name: id
			* type: unsigned integer
			* autoincremented, not null
			*/
		void appendFieldId ();
    bool appendValues ( QStringList value );
    QList<QStringList> values();
    void addForeignKey ( QString fieldName,
                         QString refTable,
                         QString refFieldName,
                         SqlTable::fkAction del = SqlTable::NoAction,
                         SqlTable::fkAction update = SqlTable::NoAction );
		/**
			* adds a foreign key reference to this table, by referencing the standard id field of the refTable
			* @param ourField the field at this table, which is a references a field of table refTable
			* @param refTable the table which contains the referenced field (standard id field)
			* @param del Action which should be on delete a key
			* @param update action which should be done, if a key is updated
			*/ 
		void addForeignKey ( SqlField *ourField, SqlTable *refTable,
                         SqlTable::fkAction del = SqlTable::NoAction,
                         SqlTable::fkAction update = SqlTable::NoAction );
		/**
			* Tries to add a renamed (form: refTableNameId, if @p newFieldName is empty, otherwise newFieldName is the name)
			* copy of the id field of @p refTable to own table.
			* Afterwards a foreignkey entry is added also
			* @note the part MOduleName_ from tablename is stripped out, cds_cds becomes cdsId and NOT cds_cdsId
			*/
		void addForeignKey ( SqlTable *refTable, QString newFieldName = QString(),
												 SqlTable::fkAction del = SqlTable::NoAction,
                         SqlTable::fkAction update = SqlTable::NoAction );
    QString sqlConstraintsString ( SqlTable::databaseType type = SqlTable::Other );
		/**
			* overlaods addIndex(field length, order), to use it without SqlFields
			*/
    void addIndex ( QString fieldName, unsigned int length = 0, Qt::SortOrder order = Qt::AscendingOrder );
		/**
			* adds an Index to field @p field
			* The Index gets the length @p length, if length is greater null.
			* The used sortorder is @p order
			*/
		void addIndex ( SqlField *field, unsigned int length = 0, Qt::SortOrder order = Qt::AscendingOrder );
    QString indizes();

  private:
    QString tableName, idFieldName;
    QList<SqlField*> myFields;
    QList<QStringList> myValues;
    QList<QStringList> constraints;
    QStringList myIndizes;
    QMultiHash<SqlTable::fkAction, QString> fkActionMap;
};

#endif // SQLTABLE_H
