/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "bookedit.h"

#include "book.h"
#include "books.h"
#include "materialpersonsmodel.h"
#include "materialperson.h"
#include "delegateforeignkey.h"
#include "msqldatabase.h"

#include <QCompleter>
#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlQueryModel>
#include <QSqlError>
#include <QHBoxLayout>
#include <QLabel>
#include <QSpinBox>
#include <QDateEdit>

BookEdit::BookEdit ( QWidget* parent, Qt::WindowFlags flags )
    : EditMedium ( parent, flags )
{
	manufacturerWidget->setLabelText(tr("Publisher:"));

	QHBoxLayout *hL = new QHBoxLayout();
	hL->addWidget(prepareLabel(tr("Edition:")));
	editionEdit = new QSpinBox(this);
	hL->addWidget(editionEdit);
	editionEdit->setRange(0, 999);
	insertLayout(hL, metaDataTab, titleWidget);
	
	hL = new QHBoxLayout();
	hL->addWidget(prepareLabel(tr("Pub. Place:")));
	pubPlaceEdit = new QLineEdit(this);
	hL->addWidget(pubPlaceEdit);
	insertLayout(hL, metaDataTab);
	
	hL = new QHBoxLayout();
	hL->addWidget(prepareLabel(tr("Pub. Date:")));
	pubDateEdit = new QDateEdit(this);
	pubDateEdit->setDisplayFormat(tr("MM/yyyy"));
	hL->addWidget(pubDateEdit);
	insertLayout(hL, metaDataTab);
	
	hL = new QHBoxLayout();
	hL->addWidget(prepareLabel(tr("Pages:")));
	pagesEdit = new QSpinBox(this);
	pagesEdit->setRange(0, 10000);
	hL->addWidget(pagesEdit);
	insertLayout(hL, metaDataTab);
}

void BookEdit::loadData(Medium* medium)
{
	Book *editedBook = qobject_cast<Book*>(medium);
	if(!editedBook)
		return;
	
	pagesEdit->setValue ( editedBook->pages() );
	editionEdit->setValue ( editedBook->edition() );
	pubDateEdit->setDate ( editedBook->publicationDate() );
	pubPlaceEdit->setText ( editedBook->publicationPlace() );

  MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return;
  
	QSqlQuery query;
	// fetching default roleId
	query.exec ( QString ( "SELECT id FROM materialpersonsroles WHERE name='%1'" ).arg(tr("Author")));
	if(query.first())
		personsWidget->setDefaultRole(query.record().value("id").toUInt());

	// adding auto completion stuff
	QSqlQueryModel *model = new QSqlQueryModel ( this );
	model->setQuery ( "SELECT publicationPlace FROM " + Books::group() + "_books WHERE publicationPlace!='' GROUP BY publicationPlace" );
	QCompleter *completer = new QCompleter ( model, this );
	completer->setCaseSensitivity ( Qt::CaseInsensitive );
	completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
	pubPlaceEdit->setCompleter ( completer );

	if(closeDB)
		db.close();
}

bool BookEdit::prepareSave(Medium* medium)
{
	Book *editedBook = qobject_cast<Book*>(medium);
	if(!editedBook)
		return false;

  editedBook->setPages ( pagesEdit->value() );
  editedBook->setEdition ( editionEdit->value() );
  editedBook->setPublicationDate ( pubDateEdit->date() );
  editedBook->setPublicationPlace ( pubPlaceEdit->text() );
	return true;
}

#include "bookedit.moc"
