/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "movieedit.h"
#include "movieedit.moc"

#include "movie.h"
#include "movies.h"
#include "editdevices.h"
#include <materialpersonsmodel.h>
#include <materialperson.h>
#include <delegateforeignkey.h>
#include <msqldatabase.h>
#include <searchbutton.h>

#include <QCompleter>
#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlQueryModel>
#include <QSqlError>
#include <QHBoxLayout>
#include <QLabel>
#include <QDateEdit>
#include <QComboBox>

MovieEdit::MovieEdit ( QWidget* parent, Qt::WindowFlags flags )
    : EditMedium ( parent, flags )
{
	manufacturerWidget->setLabelText(tr("Studio:"));

	QHBoxLayout *hL = new QHBoxLayout();
	hL->addWidget(prepareLabel(tr("Pub. Date:")));
	pubDateEdit = new QDateEdit(this);
	pubDateEdit->setObjectName("pubDateEdit");
	pubDateEdit->setDisplayFormat(tr("MM/yyyy"));
	hL->addWidget(pubDateEdit);
	insertLayout(hL, metaDataTab);
	
	hL = new QHBoxLayout();
	hL->addWidget(prepareLabel(tr("Device:")));
	deviceEdit = new QComboBox(this);
	deviceEdit->setObjectName("deviceEdit");
	hL->addWidget(deviceEdit);
	SearchButton *button = new SearchButton(this);
	button->setObjectName("deviceSearchButton");
	hL->addWidget(button);
	connect(button, SIGNAL(clicked()), this, SLOT(newDevice()));
	button->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	button->setMaximumHeight(30);
	insertLayout(hL, metaDataTab);
	deviceModel = new QSqlQueryModel(this);
	deviceEdit->setModel(deviceModel);

	tabOrder(0, layout());
}

void MovieEdit::loadData(Medium* medium)
{
	editedMovie = qobject_cast<Movie*>(medium);
	if(!editedMovie)
		return;
	
	pubDateEdit->setDate ( editedMovie->publicationDate() );


  MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return;
  
	QSqlQuery query;
	// fetching default roleId
	query.exec ( QString ( "SELECT id FROM materialpersonsroles WHERE name='%1'" ).arg(tr("Actor")));
	if(query.first())
		personsWidget->setDefaultRole(query.record().value("id").toUInt());

	// adding auto completion stuff
	deviceModel->setQuery("SELECT id, name FROM "+ Movies::group() +"_devices ORDER BY name");
	deviceEdit->setModelColumn(deviceModel->record().indexOf("name"));
	deviceEdit->setCurrentIndex(deviceEdit->findText(editedMovie->deviceName()));
	if(deviceEdit->currentText().isEmpty())
		deviceEdit->setCurrentIndex(0);

	if(closeDB)
		db.close();
}

bool MovieEdit::prepareSave(Medium* medium)
{
	if(!editedMovie)
		return false;

  editedMovie->setPublicationDate ( pubDateEdit->date() );
  editedMovie->setDevice ( deviceEdit->currentText() );
	return true;
}

void MovieEdit::newDevice()
{
	EditDevices edit(editedMovie->deviceName(), this);
	connectByPass(&edit);
	if(edit.exec() != QDialog::Accepted)
		return;

	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(db.isOpen()){
		deviceModel->setQuery(deviceModel->query());
		if(closeDB)
			db.close();
	}
	editedMovie->setDevice(edit.selectedName(), false);
	editedMovie->setDevice(edit.selectedId(), false);
	deviceEdit->setCurrentIndex(deviceEdit->findText(editedMovie->deviceName()));
}
