/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef TRACK_H
#define TRACK_H

#include "medium.h"

class TracksModel;

#include <QDate>
#include <QTime>

/**
  * Represants a Track
	* @todo allow a null manufacturer for a track?
	*/
class Track : public Medium
{
	Q_OBJECT
  public:
    Track ( Medium *parent = 0 );
 
		Medium *newMedium();
    bool select( QSqlRecord record);
    bool select ( unsigned int searchId, unsigned int userId = 0 );
    bool store(bool addOwner = false, bool startTransaction = true);
		bool setValue ( QString attribute, QVariant value );
    QVariant value ( QString attribute, int role );
    EditMedium *editWidget(MWidget* parent = 0);

    void setLength ( QTime duration );
    QTime length();
		
		Medium *copyMedium();
		bool exportMedium(QTextTable* table, Medium::ExportFormat format, bool titleOnly = false);
		/**
			* returns true if the track could be prepared
			* @param newDB we must allow the caller to gave as a database, to use the same db cause of transactions
			*/
		bool prepareTrack();
		
		void setDisc(unsigned int num);
		unsigned int disc();
		void setTrackNumber(unsigned int num);
		unsigned int trackNumber();
		
		// handle tracks
		/**
			* @see TracksModel for explanations
			*/
		void setAlbumMode(bool enable = true);
		bool albumMode() const;
		void setCollectionMode(bool enable = true);
		bool collectionMode() const;
		
    TracksModel *tracks();
    bool removeTrack(QModelIndex index);

		QList<ColKeyTrans> supportedAttrs();
		
  private:
    QTime myLength;
		bool myAlbumMode, myCollectionMode;
		unsigned int myDisc, myTrackNumber;
		TracksModel *myTracks;
};

#endif // TRACK_H
