/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "trackedit.h"

#include "track.h"
#include "cds.h"
#include <editgenres.h>
#include <editmanufacturers.h>
#include <materialpersonsmodel.h>
#include <materialperson.h>
#include <editmaterialpersons.h>
#include <delegateforeignkey.h>
#include <msqldatabase.h>
#include "tracksview.h"
#include "parentview.h"

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QHBoxLayout>
#include <QTimeEdit>

TrackEdit::TrackEdit ( QWidget* parent, Qt::WindowFlags flags )
    : EditMedium ( parent, flags )
{
	lowerTabWidget->removeTab(lowerTabWidget->indexOf(loanTab));
	
	tracksView = new TracksView(this);
	connectByPass(tracksView);
	lowerTabWidget->addTab(tracksView, tr("Tracks"));
	
	parentView = new ParentView(this);
	connectByPass(parentView);
	lowerTabWidget->addTab(parentView, tr("Related Media"));
	
	QHBoxLayout *ml = new QHBoxLayout();
	ml->addWidget(prepareLabel(tr("Length:")));
	lengthEdit = new QTimeEdit(this);
	ml->addWidget(lengthEdit);
	lengthEdit->setDisplayFormat("HH:mm:ss");
	insertLayout(ml, metaDataTab);
	
	manufacturerWidget->setLabelText(tr("Label:"));
}

void TrackEdit::loadData(Medium* medium)
{
	Track *editedTrack = qobject_cast<Track*>(medium);
	if ( !editedTrack )
    return;
	
	lengthEdit->setTime( editedTrack->length() );
	
	tracksView->setModel(editedTrack->tracks());
	parentView->setMedium(editedTrack);

	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if ( closeDB ){
		if(db.open())
			return;
	}

	QSqlQuery query;
	// fetching default roleId
	query.exec ( QString ( "SELECT id FROM materialpersonsroles WHERE name='%1'" ).arg(tr("Artist")));
	if(query.first())
		personsWidget->setDefaultRole(query.record().value("id").toUInt());

	if(closeDB)
		db.close();
}

bool TrackEdit::prepareSave(Medium* medium)
{
	Track *editedTrack = qobject_cast<Track*>(medium);
	if ( !editedTrack )
    return false;

	editedTrack->setLength(lengthEdit->time());
	return true;
}

#include "trackedit.moc"
