/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef TRACKSMODEL_H
#define TRACKSMODEL_H

#include "mediamodel.h"

class Track;

class TracksModel : public MediaModel
{
	Q_OBJECT

  public:
		enum Mode { Standard, Album, Collection } __attribute__ ((deprecated));
		
    TracksModel ( QueryThread *queryThread, QObject *parent = 0 );

		MediaModel *createNewModel();
		bool addMedium ( unsigned int materialsId );
		bool addMedium ( Medium *medium );
    Medium *newMedium() const;
		/**
			* this function should be use to store the model, if it is in
			* albumMode or collectionMode
			* if @p parent is inserted the first time, @p ignoreRemoved should be set to true,
			* to do nothing on tracks, which are marked for removel
			*/
		bool store(Medium *parent, bool ignoreRemoved);
		/**
			* use this function to let the model load all tracks,
			* which belongs @p parent
			* To load specific data, set @p mode to TracksModel::Album or
			* TracksModel::Collection
			*/
		bool loadTracks(Medium* parent);
    void addItemDelegates(QAbstractItemView *view);
		
		virtual Qt::ItemFlags flags ( const QModelIndex& index ) const;
				/**
			* the album mode allows you to store a disc number and a track number
			* for each track at the model (not at the database)
			* it also changes the display columns
			* it should be set to true if you use the model for instance as tracklist on a cd
			*/
		void setAlbumMode(bool enable = true);
		bool albumMode() const;
		/**
			* It should be set to true, if you use this model as a childmodel of a Track.
			* It provides the possibility to store a number of the track at the collection.
			*/
		void setCollectionMode(bool enable = true);
		bool collectionMode() const;
		
	private:
		bool storeAlbumMode(Medium *parent, bool ignoreRemoved);
		bool storeCollectionMode(Medium *parent, bool ignoreRemoved);
		
		bool myAlbumMode, myCollectionMode;
};

#endif // TRACKSMODEL_H
