/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "dbinit.h"

#include <minvalidator.h>

#include <QSqlQuery>
#include <QSqlError>
#include <QSqlRecord>

DBInit::DBInit ( QObject *parent,
								 QString username, QString host,
								 QString database, QString driver,
								 QString password )
	: SqlInit ( parent, username, host, database, driver, password )
{
	setObjectName("DBinit");
  init();
}

void DBInit::scheme()
{
  currentVersion = 9;
  thisName = "meganizer";
  moduleLabel = "Meganizer";

	// addTable appends the table to the table list and creates a id field,
	// if not called with false at the end
	SqlTable *table =  addTable ( "dbmoduleversions" );

  SqlField *field = table->appendField ( "module", SqlField::varchar, true );
  field->length ( 45 );
  table->addIndex ( field );

  field = table->appendField ( "moduleLabel", SqlField::varchar, true );
  field->length ( 100 );

  field = table->appendField ( "version", SqlField::integer );
  field->isUnsigned ( true );
  field->zeroFill ( true );

  // users table
  SqlTable *users = addTable ( "users" );

  field = users->appendField ( "name", SqlField::varchar, true );
  field->length ( 45 );
  users->addIndex ( field );

  field = users->appendField ( "givenName", SqlField::varchar );
  field->length ( 45 );

  field = users->appendField ( "userName", SqlField::varchar, true );
  field->length ( 45 );
  users->addIndex ( field );

  field = users->appendField ( "email", SqlField::varchar );
  field->length ( 120 );

  // material type
  table = addTable ( "materialtypes" );

  field = table->appendField ( "name", SqlField::varchar, true );
  field->length ( 45 );
  table->addIndex ( field, 20 );

  field = table->appendField ( "description", SqlField::varchar );
  field->length ( 500 );

  // manufacturer table
  SqlTable *manuTable = addTable ( "manufacturers" );

  field = manuTable->appendField ( "name", SqlField::varchar, true );
  field->length ( 120 );
  manuTable->addIndex ( field, 20 );

  field = manuTable->appendField ( "street", SqlField::varchar );
  field->length ( 50 );

  field = manuTable->appendField ( "number", SqlField::varchar );
  field->length ( 50 );

  field = manuTable->appendField ( "city", SqlField::varchar );
  field->length ( 50 );

  field = manuTable->appendField ( "postalCode", SqlField::integer );
  field->isUnsigned ( true );
	
	field = manuTable->appendField ( "postOfficeBox", SqlField::integer );
	field->isUnsigned( true );

  field = manuTable->appendField ( "country", SqlField::varchar );
  field->length ( 50 );

  field = manuTable->appendField ( "website", SqlField::varchar );
  field->length ( 120 );

  field = manuTable->appendField ( "email", SqlField::varchar );
  field->length ( 120 );

  field = manuTable->appendField ( "phone", SqlField::varchar );
  field->length ( 50 );

  field = manuTable->appendField ( "fax", SqlField::varchar );
  field->length ( 50 );

  // currencies table
  table = addTable ( "currencies" );

  field = table->appendField ( "name", SqlField::varchar, true );
  field->length ( 120 );

  field = table->appendField ( "shortName", SqlField::varchar, true );
  field->length ( 5 );
  table->addIndex ( field );

  field = table->appendField ( "toReferenceCurrency", SqlField::Double );
  field->isUnsigned ( true );

  field = table->appendField ( "isReference", SqlField::boolean );

  QStringList curDef;
  curDef << "" << "Euro" << "EUR" << "1" << "1";
  table->appendValues ( curDef );

  table = addTable ( "genres" );

  field = table->appendField ( "name", SqlField::varchar, true );
  field->length ( 50 );
  table->addIndex ( field, 10 );

  field = table->appendField ( "description", SqlField::varchar );
  field->length ( 500 );

  // material metadata table
	SqlTable *materials = addTable ( "materials" );

  field = materials->appendField ( "materialtypesId", SqlField::integer, true );
  field->isUnsigned ( true );
  materials->addForeignKey ( field->name(), "materialtypes", "id" );

  field = materials->appendField ( "manufacturersId", SqlField::integer, true );
  field->isUnsigned ( true );
  materials->addForeignKey ( field, manuTable );

  field = materials->appendField ( "genresId", SqlField::integer );
  field->isUnsigned ( true );

  field = materials->appendField ( "title", SqlField::varchar, true );
  field->length ( 120 );
  materials->addIndex ( field, 20 );

  field = materials->appendField ( "listPrice", SqlField::Double );
  field->isUnsigned ( true );

  field = materials->appendField ( "listPriceCurrenciesId", SqlField::integer, true );
  field->isUnsigned ( true );
  materials->addForeignKey ( field->name(), "currencies", "id" );
  
  field = materials->appendField ( "description", SqlField::text);
  
  field = materials->appendField("image", SqlField::blob);

  // user owns material
  table = addTable ( "userownmaterials" );
  table->addForeignKey ( materials );
	table->addForeignKey ( users );

  field = table->appendField ( "boughtPrice", SqlField::Double, true );
  field->isUnsigned ( true );
	field->defaultValue(0);

	field = table->appendField ( "boughtPriceCurrenciesId", SqlField::integer, true );
  field->isUnsigned ( true );
  table->addForeignKey ( field->name(), "currencies", "id" );
	
	field = table->appendField ( "evaluation", SqlField::integer );
  field->isUnsigned ( true );

  field = table->appendField ( "wish", SqlField::boolean, true );
	field->defaultValue(0);
	
	field = table->appendField ( "giftOnly", SqlField::boolean, true );
	field->defaultValue(0);
	
	field = table->appendField ( "literatureOnly", SqlField::boolean, true );
	field->defaultValue(0);

	// MIN stuff
	SqlTable *minTypes = addTable("mintypes");
	
	field = minTypes->appendField ( "name", SqlField::varchar, true );
	field->length(200);
	minTypes->addIndex(field, 5);
	
	field = minTypes->appendField ( "description", SqlField::text );
	
	MINValidator *minValidator = new MINValidator(this);
	QStringList types = minValidator->supportedTypes();
	foreach(QString type, types){
		QStringList defType;
		defType << "" << type << minValidator->descriptionToType(type);
		minTypes->appendValues ( defType );
	}
	delete minValidator;
	
	table = addTable("materialhasmins");
	table->addForeignKey(materials);
	table->addForeignKey(minTypes);
	
	field = table->appendField("min", SqlField::varchar, true);
	field->length(20);
	table->addIndex(field, 13);
	
  // loan stuff
	table = addTable ( "loanstati" );
	
	field = table->appendField( "name", SqlField::varchar, true );
	field->length(50);
	
	QStringList insert;
	insert << "" << "Loan Requested";
  table->appendValues ( insert );
  insert.clear();
  insert << "" << "Loan Denied";
  table->appendValues ( insert );
  insert.clear();
  insert << "" << "Loaned";
  table->appendValues ( insert );
  insert.clear();
  insert << "" << "Return Requested";
  table->appendValues ( insert );
	insert.clear();
	insert << "" << "Returned";
  table->appendValues ( insert );
	insert.clear();
	insert << "" << "Request Canceled";
  table->appendValues ( insert );
	
  table = addTable ( "loanedmaterials" ) ;

  table->addForeignKey ( materials );
  table->addForeignKey ( users, "ownerId" );
  table->addForeignKey ( users, "recipientId" );

  field = table->appendField ( "status", SqlField::integer, true );
  field->isUnsigned ( true );
  table->addForeignKey ( field->name(), "loanstati", "id" );

	field = table->appendField ( "loanedUntil", SqlField::timestamp, true);
	field->defaultValue(0); // to prevent this column from being auto updated
	
	field = table->appendField( "lastChange", SqlField::timestamp, true);
	field->defaultValue("CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP");

  // material persons are persons with a relation to a material
  // for example an author, a producer, an actor or artist
  table = addTable ( "materialpersons" );

  field = table->appendField ( "name", SqlField::varchar, true );
  field->length ( 120 );
  table->addIndex ( field, 20 );

  field = table->appendField ( "givenName", SqlField::varchar );
  field->length ( 120 );

  // materialPersonRoles
  // roles of materialPersons such like is an actor, is the producer, ...
  table = addTable ( "materialpersonsroles" );

  field = table->appendField ( "name", SqlField::varchar, true );
  field->length ( 50 );
  table->addIndex ( field, 10 );

  field = table->appendField ( "description", SqlField::varchar );
  field->length ( 500 );

  // combining material, persons and roles
  table = addTable ( "materialhaspersons" );

  table->addForeignKey ( materials );

  field = table->appendField ( "materialPersonsId", SqlField::integer, true );
  field->isUnsigned ( true );
  table->addForeignKey ( field->name(), "materialpersons", "id" );

  field = table->appendField ( "materialPersonsRolesId", SqlField::integer, true );
  field->isUnsigned ( true );
  table->addForeignKey ( field->name(), "materialpersonsroles", "id" );

  // material has materials a kind of "Stückliste"
  // e.g. can be used to add material track to a material album
  // 			and this way a CD with track list can be stored
  table = addTable ( "materialhasmaterials" );
  table->addForeignKey ( materials );
	table->addForeignKey ( materials, "hasMaterialsId" );

	table = addTable ( "projects" );
	
	field = table->appendField ( "name", SqlField::varchar, true );
	field->length(120);
	
	table->addForeignKey ( users );
	table->addForeignKey ( materials, "resultId" );
	
	field = table->appendField ( "description", SqlField::varchar, true );
	field->length( 500 );
	
	table = addTable ( "projectscollection" );
	
	field = table->appendField( "projectsId", SqlField::integer, true );
	field->isUnsigned( true );
	table->addForeignKey ( field->name(), "projects", "id" );
	
	table->addForeignKey ( materials );
	
	field = table->appendField ( "comment", SqlField::varchar, true );
	field->length( 500 );
	
	table = addTable ( "gifts" );
	
	table->addForeignKey ( materials );
	table->addForeignKey ( users, "donerId" );
	table->addForeignKey ( users, "presenteeId" );
	
	field = table->appendField ( "finished", SqlField::timestamp, true);
	field->defaultValue(0); // to prevent this column from being auto updated

	SqlTable *keywords = addTable ( "keywords" );
	field = keywords->appendField( "keyword", SqlField::varchar, true );
	field->length(100);
	keywords->addIndex(field, 10);

	table = addTable ( "materialhaskeywords" );
	table->addForeignKey( materials );
	table->addForeignKey( keywords );
	
  QStringList view;
  view.append ( "viewmaterials" );
  view.append ( "SELECT  mats.id, mats.materialtypesId, mats.manufacturersId,"
								"  mats.genresId, mats.title, mats.listPrice, mats.listPriceCurrenciesId,"
								"  mats.description, mats.image,"
								"  user.usersId, user.boughtPrice, user.boughtPriceCurrenciesId,"
								"  user.evaluation, user.wish, user.giftOnly, user.literatureOnly"
                "  FROM materials mats LEFT JOIN userownmaterials user ON mats.id=user.materialsId" );
  views.append ( view );
	
	view.clear();
	view.append ( "viewmaterialpersons" );
	view.append ( "SELECT mats.*, matper.materialPersonsId, persons.name, persons.givenName"
								"  FROM viewmaterials mats"
								"  LEFT JOIN materialhaspersons matper ON mats.id=matper.materialsId"
								"  LEFT JOIN materialpersons persons ON matper.materialPersonsId=persons.id"
								"  WHERE matper.materialsId<>0" );
	views.append ( view );
	
	view.clear();
	view.append ( "viewprojectmaterials" );
	view.append ( "SELECT proj.id AS rowId, proj.projectsId, proj.comment, mats.*"
								"  FROM projectscollection proj"
								"  LEFT JOIN viewmaterials mats ON proj.materialsId=mats.id");
	views.append ( view );
	
	view.clear();
	view.append ( "viewgiftmaterials" );
	view.append ( "SELECT mats.*, gifts.donerId, gifts.presenteeId, gifts.finished"
								"  FROM viewmaterials mats"
								"  LEFT JOIN gifts ON gifts.materialsId=mats.id AND mats.usersId=gifts.donerId");
	views.append ( view );

	view.clear();
	view.append ( "viewmaterialskeywords" );
  view.append ( "SELECT mats.id, mats.materialsId, mats.keywordsId, keywords.keyword"
								"  FROM materialhaskeywords mats"
								"  LEFT JOIN keywords ON mats.keywordsId=keywords.id");
	views.append ( view );
	
	// register update routinges
	addUpdateRoutine(4, &DBInit::updateFrom3To4);
	addUpdateRoutine(6, &DBInit::updateFrom5To6);
	addUpdateRoutine(7, &DBInit::updateFrom6To7);
	addUpdateRoutine(8, &DBInit::updateFrom7To8);
}

bool DBInit::updateFrom3To4 (QString thisName, QList< SqlTable* > tables, QString& errorString )
{
	// 1. delete useless table userhasmaterials
	QSqlQuery query;
	if(!query.exec("DROP TABLE userhasmaterials")){
		errorString.append ( query.lastError().text() );

		return false;
	}
	return true;
}

bool DBInit::updateFrom5To6 (QString thisName, QList<SqlTable*> tables, QString &errorString )
{
	QSqlQuery query;
	if(!query.exec("UPDATE userownmaterials SET giftOnly=0, literatureOnly=0")){
		errorString.append ( query.lastError().text() );

		return false;
	}
	
	if(!query.exec("UPDATE userownmaterials SET wish=0 WHERE wish<>1")){
		errorString.append ( query.lastError().text() );

		return false;
	}
	
	// alter table fields
	if(!query.exec("ALTER TABLE userownmaterials ALTER wish SET DEFAULT 0")){
		errorString.append ( query.lastError().text() );

		return false;
	}
	if(!query.exec("ALTER TABLE userownmaterials ALTER giftOnly SET DEFAULT 0")){
		errorString.append ( query.lastError().text() );

		return false;
	}
	if(!query.exec("ALTER TABLE userownmaterials ALTER literatureOnly SET DEFAULT 0")){
		errorString.append ( query.lastError().text() );

		return false;
	}
	
	return true;
}

bool DBInit::updateFrom6To7(QString thisName, QList< SqlTable* > tables, QString& errorString)
{
	// alter projectscollection table
	QSqlQuery query;
	if(!query.exec("ALTER TABLE projectscollection CHANGE COLUMN comment comment varchar(500) NULL")){
		errorString.append( query.lastError().text() + "\n" + query.lastQuery() );
		return false;
	}
	
	// move already present eans to new materialhasmins table
	MINValidator *validator = new MINValidator();
	if(!validator->isSupported("EAN")){
		errorString.append(tr("Could not find EAN-Validator!\n"
													"Loaded Validators: %1")
												.arg(validator->supportedTypes().join(", "))
											);
		delete validator;
		return false;
	}
	delete validator;
	
	query.prepare("SELECT id FROM mintypes WHERE name=:name");
	query.bindValue(":name", "EAN");
	query.exec();
	if(!query.first())
		return false;
	if(query.size() != 1)
		return false;
	unsigned int eanId = query.record().value("id").toUInt();
	if(eanId <= 0){
		errorString.append(tr("Could not find id of MIN EAN\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	
	query.exec("SELECT id, ean FROM materials WHERE ean IS NOT NULL AND ean<>''");
	QSqlQuery insertQuery;
	while(query.next()){
		insertQuery.prepare("INSERT INTO materialhasmins (materialsId, mintypesId, min) VALUES(:materialsId, :mintypesId, :min)");
		insertQuery.bindValue(":materialsId", query.record().value("id").toUInt());
		insertQuery.bindValue(":mintypesId", eanId);
		insertQuery.bindValue(":min", query.record().value("ean").toString());
		if(!insertQuery.exec()){
			errorString.append(tr("Could not move EAN's to new table!\n%1\n%2")
														.arg(query.lastError().text()).arg(query.lastQuery()));
			return false;
		}
	}
	
	// alter materials table
	if(!query.exec("ALTER TABLE materials DROP COLUMN ean")){
		errorString.append ( query.lastError().text() );
		return false;
	}
	
	return true;
}

bool DBInit::updateFrom7To8(QString thisName, QList<SqlTable*> tables, QString& errorString)
{
	QSqlQuery query;
	if(!query.exec("ALTER TABLE loanstati DROP COLUMN action")){
		errorString.append ( query.lastError().text() );
		return false;
	}
	if(!query.exec("ALTER TABLE loanstati DROP COLUMN description")){
		errorString.append ( query.lastError().text() );
		return false;
	}
	
	foreach(SqlTable *table, tables){
		if(table->name() == "loanstati"){
			// first, delete all not needed stati
			query.prepare("DELETE FROM loanstati WHERE id > :id");
			query.bindValue(":id", table->values().size());
			if(!query.exec()){
				errorString.append( query.lastError().text() +"\n"+ query.lastQuery() );
				return false;
			}
			// rename the stati
			if(table->values().size() != 6)
				return false;
			
			for(int  i = 0; i < table->values().size(); i++){
				query.prepare("UPDATE loanstati SET name=:name WHERE id=:id");
				query.bindValue(":name", table->values()[i][1]);
				query.bindValue(":id", i + 1);
				if(!query.exec()){
					errorString.append( query.lastError().text() +"\n"+ query.lastQuery() );
					return false;
				}
			}
			return true;
		}
	}
	return false;
}

#include "dbinit.moc"
