/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "meganizerconfig.h"

#include "dependencies.h"

#include <QDir>
#include <QDomDocument>
#include <QDomNodeList>

Dependencies::Dependencies ( QWidget* parent, Qt::WFlags fl )
		: MDialog ( parent, fl )
{
	setupUi ( this );
}

Dependencies::~Dependencies()
{
	close();
}

void Dependencies::init()
{
	QDir dir ( QString ( SHAREDIR ) + "/depends" );

	if ( dir.exists() ) {
		QStringList filter;
		filter.append ( "*.xml" );
		QStringList newFiles = dir.entryList ( filter, QDir::Files | QDir::Readable );
		foreach ( QString file, newFiles ) {
			addApp ( dir.path() + "/" + file );
		}
	}
	
	model.setHeaderData(0, Qt::Horizontal, tr("Name"));
	model.setHeaderData(1, Qt::Horizontal, tr("Value"));
	dependencyView->setModel ( &model );

// 	dependencyView->setHeaderHidden ( true );
	dependencyView->resizeColumnToContents ( 0 );
	dependencyView->resizeColumnToContents ( 1 );
}

void Dependencies::addApp ( QString file )
{
	QFile xmlFile ( file );

	// convert Data into a qdomdocument
	QDomDocument result;
	QDomNodeList node;
	QString errorStr;
	int errorLine;
	int errorColumn;

	if ( !result.setContent ( &xmlFile, true, &errorStr, &errorLine, &errorColumn ) ) {
		emit information ( tr ( "Can not convert data from %1 into an XML-DOM-Document. \n Error at line %2, Column %3: %4" )
		                   .arg ( xmlFile.fileName() )
		                   .arg ( errorLine )
		                   .arg ( errorColumn )
		                   .arg ( errorStr ) );
		return;
	}

	// first check wether the request was valid
	QDomElement root = result.documentElement();

	if ( root.tagName() != "Dependencies" ) {
		emit information ( tr ( "No valid document" ) );
		return;
	}

	node = result.elementsByTagName ( "app" );

	if ( node.size() != 1 ) {
		emit information ( tr ( "No app found!" ) );
		return;
	}

	model.setColumnCount ( 2 );

	QStandardItem *appItem = new QStandardItem ( node.item ( 0 ).toElement().text() );
	model.appendRow ( appItem );

	QDomElement item = root.firstChildElement();

	while ( !item.isNull() ) {
		if ( item.tagName() == "dependency" ) {
			addDependencies ( item, appItem );
		}

		item = item.nextSiblingElement();
	}
}

void Dependencies::addDependencies ( QDomElement dependency, QStandardItem *item )
{
	// add dependency
	QDomElement child = dependency.firstChildElement();
	QStandardItem *depName = new QStandardItem();

	while ( !child.isNull() ) {
		if ( child.tagName() == "name" ) {
			depName->setText ( child.text() );
			item->appendRow ( depName );
		}

		if ( child.tagName() == "version" ) {
			QList<QStandardItem*> tmp;
			tmp.append ( new QStandardItem ( tr ( "Version" ) ) );
			tmp.append ( new QStandardItem ( child.text() ) );
			depName->appendRow ( tmp );
		}

		if ( child.tagName() == "url" ) {
			QList<QStandardItem*> tmp;
			tmp.append ( new QStandardItem ( tr ( "Website" ) ) );
			tmp.append ( new QStandardItem ( child.text() ) );
			depName->appendRow ( tmp );
		}

		if ( child.tagName() == "dependency" ) {
			addDependencies ( child, depName );
		}

		child = child.nextSiblingElement();
	}
}

#include "dependencies.moc"
