/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "meganizerconfig.h"

#include "mediendb.h"

#include <QApplication>
#include <QTranslator>
#include <QSettings>
#include <QDir>
#include <QResource>
#include <QLocale>
#include <QLibraryInfo>

/**
	* @mainpage Meganizer - Media Organizer
	*
	* @section sec1 About
	*
	* Meganizer - Media Organizer is a tool to manage existing media collections. It is open to integrate
	* new media types with help of  plugins
	*
	* @subsection sec11 Project Structure
	*
	* The project is structered in a shared library called libmeganizer, which provide basic classes,
	* which are used by the main application meganizer and the MediaPlugins and SearchPlugins.
	* The application and the library itself contains only stuff, which is needed by all types of medias.
	* The application contains the main window and provides some gui stuff.
	* Each specific medium type is contributed withhelp of an MediaPlugin.
	* This plugins provide all needed stuff to handle the specific mediatype.
	* SearchPlugin 's provides the possibilitie to search media meta data and may be allows to buy them too.
	* 
	* <ul>
	*  <li>Root</li>
	*  <ul>
	*    <li>doc (Contains all Documentation stuff)</li>
	*    <ul>
	*      <li>dev (Documentation related to delevopment e.g. Database model, ...)</li>
	*      <li>app (Enduser documentation)</li>
	*      <ul>
	*        <li>man (Manpages)</li>
	*        <li>manual (Usermanual in Latex, HTML, PDF and Online usage in Application)</li>
	*      </ul>
	*      <li>src (Doxygen source documentation)</li>
	*    </ul>
	*    <li>src (Contains Source files)</li>
	*    <ul>
	*      <li>LibMeganizer</li>
	*      <li>LibMeganizerTests</li>
	*      <li>Meganizer (Main Application, entry: MedienDB)</li>
	*      <li>MediaPlugin 's</li>
	*      <ul>
	*        <li>Books</li>
	*        <li>Music (CD's, Tracks, LP's, ..., entry: CDs) </li>
	*      </ul>
	*      <li>SearchPlugin 's</li>
	*      <ul>
	*        <li>Amazon</li>
	*        <li>DemoShop (Testing purposes)</li>
	*      </ul>
	*    </ul>
	*  </ul>
	* </ul>
	*
	* @section sec2 Developing
	*
	* @subsection sec21 Prepare Environment
	*
	* To make development easier, libmeganizer provides an QtDesigner plugin, which contains the needed widgets.
	*
	* @subsection sec22 Contributing Work
	*
	* If you have done some changes and you think, they may be need by other people,
	* please create a patch and send us a mail to meganizer@engsas.de, send it to the mailing list or
	* create a ticket and append the patch.
	*
  * @author Joachim Langenbach
  */
int main ( int argc, char** argv )
{
  QApplication app ( argc, argv );
	
	// QSettings organization name und application name
	// urls are setted after translations are loaded,
	// because the urls is translated
	QString shortName = "Meganizer";
	QString appTitle = shortName + " - Media Organizer";
	QString appVersion = QString ( VERSION );
	QString prodName = "EngSaS";
	QString prodMail = "info@engsas.de";
	QCoreApplication::setOrganizationName ( prodName );
	QCoreApplication::setApplicationName ( shortName );

	QSettings settings;
	settings.setValue ( "appNameShort", shortName );
	settings.setValue ( "appName", appTitle );
	settings.setValue ( "appVersion", appVersion );
	settings.setValue ( "prodName", prodName );
	settings.setValue ( "prodMail", prodMail );

  // Program start
  // try to load translation-files
	QLocale qLocale;
	QString lang = settings.value (
		"defaultLang",
		qLocale.name().left(2).toUpper()
	).toString();
  QDir dir ( QString ( SHAREDIR ) + "/lang" );

	if ( dir.exists() ) {
		QStringList filter;
		filter.append ( "*.qm" );
		QStringList newFiles = dir.entryList ( filter, QDir::Files | QDir::Readable );
		foreach ( QString file, newFiles ) {
			// Needed for the translation of the app
			// only add the translations of the choosen language
			if(file.contains(lang, Qt::CaseInsensitive)){
				QTranslator *translator = new QTranslator ( &app );
				translator->load ( dir.path() + "/" + file );
				app.installTranslator ( translator );
			}
    }
  }
  // load default translations of qt
	// this must be done only on unix, because on windows,
	// the qt-translation file is also at lang-directory
 #if not defined(Q_OS_WIN32)
	QTranslator qtTranslator;
	qtTranslator.load("qt_" + QLocale::system().name(),
	QLibraryInfo::location(QLibraryInfo::TranslationsPath));
	app.installTranslator(&qtTranslator);
#endif

  // try to load resources
  dir.setPath ( QString ( SHAREDIR ) + "/rcc" );

	if ( dir.exists() ) {
		QStringList filter;
		filter.append ( "*.rcc" );
		QStringList newFiles = dir.entryList ( filter, QDir::Files | QDir::Readable );
		foreach ( QString file, newFiles ) {
			QResource::registerResource ( dir.path() + "/" + file );
		}
	}

	QString prodWWW = QObject::tr("http://en.engsas.de");
	QString appWWW = prodWWW + "/meganizer";
		
	settings.setValue ( "appWWW", appWWW );
	settings.setValue ( "prodWWW", prodWWW );


	MedienDB *mediendb = new MedienDB(argc, argv);
	if(mediendb->shouldExit())
		return 1;
	
	mediendb->init();
	mediendb->show();
  return app.exec();
}
