/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "modulestate.h"
#include "ui_modulestate.h"

#include <QListWidgetItem>

ModuleState::ModuleState ( QList<ModuleListWidgetItem*> newProtocol, QWidget *parent ) 
	: MWidget ( parent ), ui ( new Ui::ModuleState )
{
  activeItem = 0;

  ui->setupUi ( this );

  connect ( ui->moduleList, SIGNAL ( itemClicked ( QListWidgetItem* ) ), this, SLOT ( itemClicked ( QListWidgetItem* ) ) );
  connect ( ui->loadButton, SIGNAL ( clicked() ), this, SLOT ( loadButton() ) );
  connect ( ui->unloadButton, SIGNAL ( clicked() ), this, SLOT ( unloadButton() ) );
  connect ( ui->reloadButton, SIGNAL ( clicked() ), this, SLOT ( reloadButton() ) );
  ui->loadButton->setEnabled ( false );
  ui->unloadButton->setEnabled ( false );
  protocol = newProtocol;
  convertProtocol();
}

ModuleState::~ModuleState()
{
  delete ui;
  this->close();
}

void ModuleState::setViewOnly()
{
	ui->loadButton->setHidden(true);
	ui->unloadButton->setHidden(true);
	ui->reloadButton->setHidden(true);
}

void ModuleState::changeEvent ( QEvent *e )
{
  QWidget::changeEvent ( e );
  switch ( e->type() ) {
    case QEvent::LanguageChange:
      ui->retranslateUi ( this );
      break;
    default:
      break;
  }
}

void ModuleState::itemClicked ( QListWidgetItem *item )
{
  // the dynamic cast isn't working, so we do a workaround
//  ModuleListWidgetItem *tmp = dynamic_cast<ModuleListWidgetItem *>(item);
//  if(!tmp){
//    ui->nameEdit->setText("test");
//    ui->fileEdit->setText("");
//    ui->descriptionEdit->setPlainText("");
//    ui->errorMessageEdit->setPlainText("");
//    return;
//  }

  ui->loadButton->setEnabled ( false );
  ui->unloadButton->setEnabled ( false );
  foreach ( ModuleListWidgetItem *module, protocol ) {
    if ( module->name() == item->text() ) {
      activeItem = module;
			ui->imageLabel->setPixmap(module->moduleIcon().pixmap(ui->imageLabel->size()));
      ui->nameEdit->setText ( module->name() );
      ui->fileEdit->setText ( module->file() );
      ui->descriptionEdit->setPlainText ( module->description() );
      ui->errorMessageEdit->setPlainText ( module->errorMessage() );
      if ( module->status() ) {
        ui->unloadButton->setEnabled ( true );
      }
      else {
        ui->loadButton->setEnabled ( true );
      }
      return;
    }
  }
  ui->nameEdit->setText ( "" );
  ui->fileEdit->setText ( "" );
  ui->descriptionEdit->setPlainText ( "" );
  ui->errorMessageEdit->setPlainText ( "" );
  activeItem = 0;
}

void ModuleState::loadButton()
{
  if ( !activeItem )
    return;

  emit load ( activeItem, true );
}

void ModuleState::unloadButton()
{
  if ( !activeItem )
    return;

  emit unload ( activeItem );
}

void ModuleState::reloadButton()
{
  emit reloadPlugins();
}

void ModuleState::convertProtocol()
{
  ui->moduleList->clear();
  foreach ( ModuleListWidgetItem *module, protocol ) {
		QListWidgetItem *item = module->clone();
    ui->moduleList->addItem ( item );
  }
  
  // resize list
	if(ui->moduleList->count() < 1)
		return;
	
  int maxSize = 0;
  QFontMetrics fontMetric = ui->moduleList->fontMetrics();
  for(int i = 0; i < ui->moduleList->count(); i++){
    if(maxSize < fontMetric.width(ui->moduleList->item(i)->text())){
      maxSize = fontMetric.width(ui->moduleList->item(i)->text());
    }
  }
  
	maxSize += ui->moduleList->iconSize().width();
  maxSize += 50;
  ui->moduleList->setMinimumWidth(maxSize);
	ui->moduleList->setMaximumWidth(maxSize);
}

#include "modulestate.moc"
