/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "meganizerconfig.h"

#include "resultsummary.h"

#include <mediaplugin.h>
#include <mediamodel.h>
#include <mdialog.h>
#include "mediumexport.h"
#include "shoppingcart.h"
#include "insertmanualmedia.h"
#include <mediumloandialog.h>
#include <mediumownerdialog.h>
#include <maction.h>
#include <editmedium.h>
#include <msqldatabase.h>
#include <etoolbar.h>

#include <QMenu>
#include <QSortFilterProxyModel>
#include <QSqlRecord>
#include <QSqlQuery>
#include <QInputDialog>

ResultSummary::ResultSummary ( QList< MediaPlugin* > newMediaPlugins, QWidget* parent, Qt::WindowFlags flags )
		: MWidget ( parent, flags )
{
	model = 0;
	mediaPlugins = newMediaPlugins;
	setupUi ( this );
	shoppingCart = 0;
	toolBar = 0;
}

ResultSummary::~ResultSummary()
{
	close();

	if ( model )
		delete model;

	if ( proxyModel )
		delete proxyModel;

	delete toolBar;
}

void ResultSummary::init ( ShoppingCart *shoppingCart )
{
	this->shoppingCart = shoppingCart;

#if QT_VERSION_MINOR < 6
	newAction = new MAction ( QIcon ( ":/icons/filenew.png" ),
	                          tr ( "New" ), this );
	addSelectedAction = new MAction ( tr ( "Add Selected" ), this );
	editAction = new MAction ( tr ( "Edit / Show Details" ), this );
	exportAction = new MAction ( QIcon ( ":/icons/fileexport.png" ),
	                             tr ( "Export" ), this );
#else
	newAction = new MAction ( QIcon::fromTheme ( "document-new",
	                          QIcon ( ":/icons/filenew.png" ) ),
	                          tr ( "New" ), this );
	addSelectedAction = new MAction ( QIcon::fromTheme ( "insert-object" ), tr ( "Add Selected" ), this );
	editAction = new MAction ( QIcon::fromTheme ( "document-properties" ), tr ( "Edit / Show Details" ), this );
	buyAction = new MAction ( QIcon(), tr ( "Buy" ), this );
	exportAction = new MAction ( QIcon::fromTheme ( "document-export",
	                             QIcon ( ":/icons/fileexport.png" ) ),
	                             tr ( "Export" ), this );
#endif
	buyAction = new MAction ( QIcon(), tr ( "Buy" ), this );
	loanAction = new MAction ( QIcon(), tr ( "Loaning" ), this );
	ownerAction = new MAction ( QIcon ( ":/icons/edituser.png" ), tr ( "Owner Manager" ), this );

	toolBar = new EToolBar ( this );
	toolBar->setObjectName ( "resultSummaryToolbar" );
	toolBar->addAction ( newAction );
	toolBar->addAction ( addSelectedAction );
	toolBar->addAction ( editAction );
	toolBar->addAction ( buyAction );
	toolBar->addAction ( exportAction );
	toolBar->addAction ( loanAction );
	toolBar->addAction ( ownerAction );
	emit addToolbar ( toolBar );

	addSelectedAction->setDisabled ( true );
	editAction->setDisabled ( true );
	buyAction->setDisabled ( true );
	exportAction->setDisabled(true);
	loanAction->setDisabled ( true );
	ownerAction->setDisabled ( true );

	connect ( newAction, SIGNAL ( triggered() ), this, SLOT ( addNew() ) );
	connect ( addSelectedAction, SIGNAL ( triggered() ), this, SLOT ( addSelected() ) );
	connect ( editAction, SIGNAL ( triggered() ), this, SLOT ( edit() ) );
	connect ( buyAction, SIGNAL ( triggered() ), this, SLOT ( buy() ) );
	connect ( exportAction, SIGNAL ( triggered() ), this, SLOT ( exportMedias() ) );
	connect ( loanAction, SIGNAL ( triggered() ), this, SLOT ( loanManager() ) );
	connect ( ownerAction, SIGNAL ( triggered() ), this, SLOT ( ownerManager() ) );
	connect ( tableView, SIGNAL ( clicked ( QModelIndex ) ), this, SLOT ( indexChanged ( QModelIndex ) ) );
	connect ( tableView, SIGNAL ( doubleClicked ( QModelIndex ) ), this, SLOT ( indexChanged ( QModelIndex ) ) );
	connect ( tableView, SIGNAL ( customContextMenuRequested ( QPoint ) ), this, SLOT ( contextMenu ( QPoint ) ) );

	tableView->setContextMenuPolicy ( Qt::CustomContextMenu );
	tableView->setSortingEnabled ( true );

	proxyModel = new QSortFilterProxyModel ( tableView );
	proxyModel->setDynamicSortFilter ( true );
	tableView->setModel ( proxyModel );
}

void ResultSummary::setResultset(MediaModel* newModel)
{
	indexChanged(QModelIndex());
	// delegates must be deleted, before
	// the proxyModel is resetted, because
	// the columnCount is needed!
	tableView->deleteDelegates();
	proxyModel->setSourceModel(0);
	if ( model )
		delete model;
	model = 0;

	model = newModel;
	
#if QT_VERSION_MINOR < 5
	exportAction->setDisabled ( true );
#else
	if ( model->rowCount() < 1 )
		exportAction->setDisabled ( true );
#endif

	if ( model ) {
		proxyModel->setSourceModel ( model );
		model->configureView ( tableView );
		model->hideAndResizeColumns(tableView);
	}

	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if ( closeDB )
		db.open();

	if ( db.isOpen() ) {
		QSqlQuery query;
		query.exec ( QString ( "SELECT id FROM users WHERE username = '%1'" ).arg ( db.userName() ) );

		if ( query.first() )
			myUserId = query.record().value ( "id" ).toUInt();

		if ( closeDB )
			db.close();
	}
}

void ResultSummary::save()
{
	// save unchanged changes
	InsertManualMedia *insertMedia = new InsertManualMedia ( this );
	connectByPass ( insertMedia );
	// 	insertMedias->setUnsavedOrigins(Medium::Manual); (It's the default actually)
	insertMedia->setupMedia ( model );
	insertMedia->exec();
	delete insertMedia;
}

void ResultSummary::closeEvent ( QCloseEvent *event )
{
	save();
	MWidget::closeEvent ( event );
}

void ResultSummary::addNew()
{
	Medium *medium = 0;
	// may be it's the standard MediaModel,
	// in that case ask for a special medium type

	if ( model->name() == "Model" ) {
		QMultiMap<QString, MediaPlugin*> mapTypes;
		foreach ( MediaPlugin *plugin, mediaPlugins ) {
			QStringList types = plugin->supportedMaterialTypes();
			foreach ( QString type, types ) {
				mapTypes.insert ( type, plugin );
			}
		}

		bool check = false;
		QString type = QInputDialog::getItem ( this, tr ( "Select Mediatype" ), tr ( "Type:" ), mapTypes.uniqueKeys(), 0, false, &check );

		if ( !check )
			return;

		MediaPlugin *plugin = mapTypes.value ( type );

		MediaModel *typeModel = plugin->newModel ( type );

		medium = typeModel->newMedium();

		delete typeModel;
	} else {
		medium = model->newMedium();
	}

	if ( !medium ) {
		emit error ( tr ( "Could not add medium" ) );
		return;
	}

	EditMedium *widget = medium->editWidget(this);
	if ( !widget ) {
		emit error ( tr ( "Could no get edit wigdet" ) );
		return;
	}
	connectByPass ( widget );
	widget->setNewMode(model, medium);

	if ( widget->exec() != QDialog::Accepted )
		emit error ( tr ( "Could not add medium!" ) );
	
	delete widget;
}

void ResultSummary::addSelected()
{
	Medium *medium = model->mediumFromIndex ( curIndex );

	if ( !medium ) {
		emit error ( tr ( "Please select a medium" ) );
		return;
	}

	// medium is already in database, so the only thing we must do is
	// to store it with current user
	if(!medium->edit(EditMedium::Add, this)){
		emit error ( tr ( "Could not store medium" ) );
		return;
	}

	indexChanged ( QModelIndex() );

	model->reload();
}

void ResultSummary::edit()
{
	if ( !model )
		return;

	Medium *medium = model->mediumFromIndex ( curIndex );

	if ( !medium ) {
		emit error ( tr ( "Please select a medium" ) );
		return;
	}

	if(!medium->edit(EditMedium::Edit, this)){
		emit error ( tr ( "Could not store changes" ) );
		return;
	}
	
	// must be made invalid, because all model indizes are invalid after reload
	indexChanged ( QModelIndex() );
	model->reload();
}

void ResultSummary::buy()
{
	if ( !curIndex.isValid() )
		return;

	if ( !model )
		return;

	Medium *medium = model->mediumFromIndex ( curIndex );

	if ( !medium )
		return;

	if ( !shoppingCart )
		return;

	shoppingCart->addMedium ( medium );
}

void ResultSummary::indexChanged ( QModelIndex index )
{
	if ( !model )
		return;

	if ( !proxyModel )
		return;

	curIndex = proxyModel->mapToSource ( index );

	// change button state
	addSelectedAction->setEnabled ( false );

	editAction->setEnabled ( false );

	buyAction->setEnabled ( false );

	exportAction->setEnabled ( false );

	loanAction->setEnabled ( false );

	ownerAction->setEnabled ( false );

#if QT_VERSION_MINOR > 4
	if ( model->rowCount() > 1 )
		exportAction->setEnabled ( true );
#endif

	if ( curIndex.isValid() ) {
		Medium *medium = model->mediumFromIndex ( curIndex );

		if ( !medium )
			return;

		buyAction->setEnabled ( true );

		if ( medium->id() == 0 ) {
			// Medium is not in database, so disable edit button
			addSelectedAction->setEnabled ( true );
		} else {
			// only mediums which are not our own, can be added to our inventory
			// our own medias can only be shown
			if ( medium->user() != myUserId )
				addSelectedAction->setEnabled ( true );

			editAction->setEnabled ( true );

			loanAction->setEnabled ( true );

			ownerAction->setEnabled ( true );
		}
	}
}

void ResultSummary::doubleClicked ( QModelIndex index )
{
	addSelected();
}

void ResultSummary::contextMenu ( QPoint pos )
{
	QModelIndex index = tableView->indexAt ( pos );
	indexChanged ( index );

	QMenu *menu = new QMenu();
//   menu->setTitle(title);

	menu->addAction ( newAction->actionWithText() );
	menu->addAction ( addSelectedAction->actionWithText() );
	menu->addAction ( editAction->actionWithText() );
	menu->addAction ( buyAction->actionWithText() );
	menu->addAction ( exportAction->actionWithText() );
	menu->addAction ( loanAction->actionWithText() );
	menu->addAction ( ownerAction->actionWithText() );
	menu->exec ( QCursor::pos() );
	delete menu;
}

void ResultSummary::exportMedias()
{
	if ( !model )
		return;
	
	if(!tableView)
		return;
	
	MediumExport *medExp = new MediumExport ( this );

	connectByPass ( medExp );
	
	QModelIndexList indexes;
	if(tableView->selectionModel())
		indexes = tableView->selectionModel()->selectedRows();
	medExp->exportMedium ( model, indexes );

	medExp->deleteLater();
}

void ResultSummary::loanManager()
{
	if ( !model )
		return;

	MediumLoanDialog *loanDialog = new MediumLoanDialog ( model->mediumFromIndex ( curIndex ), this );

	connectByPass ( loanDialog );

	loanDialog->exec();

	delete loanDialog;
}

void ResultSummary::ownerManager()
{
	if ( !model )
		return;

	MediumOwnerDialog *ownerDialog = new MediumOwnerDialog ( model->mediumFromIndex ( curIndex ), this );

	connectByPass ( ownerDialog );

	ownerDialog->exec();

	delete ownerDialog;
}

#include "resultsummary.moc"
