/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "meganizerconfig.h"

#include "searchwidget.h"

#include <mediaplugin.h>
#include <searchplugin.h>
#include "resultsummary.h"
#include <mediamodel.h>
#include <searchdetailswidget.h>
#include <sqlconstraint.h>
#include <msqldatabase.h>
#include <minvalidator.h>

#include <QSettings>
#include <QSqlQuery>
#include <QDockWidget>
#include <QAction>
#include <QTabWidget>
#include <QSizePolicy>
#include <QCompleter>
#include <QSqlQueryModel>

SearchWidget::SearchWidget ( QWidget* parent, Qt::WindowFlags flags )
    : MWidget ( parent, flags )
{
  ownUserId = 0;
	genreModel = 0;
	manuModel = 0;
	minValidator = 0;
  
	queryThread = new QueryThread(this);
	connect(queryThread, SIGNAL(started()), this, SLOT(setupQueryThread()));
	queryThread->start();
	
	setObjectName("SearchWidget");
  setupUi ( this );
	
// #if QT_VERSION_MINOR < 6
	searchButton->setIcon(QIcon(":/icons/filefind.png"));
// #else
// 	searchButton->setIcon(QIcon::fromTheme("edit-find",
// 																				 QIcon(":/icons/filefind.png")));
// #endif
	
  connect ( searchButton, SIGNAL ( clicked() ), this, SLOT ( search() ) );
	
	connect ( titleEdit, SIGNAL( returnPressed() ), this, SLOT( search() ) );
	connect ( personEdit, SIGNAL( returnPressed()), this, SLOT( search() ) );
	connect ( eanEdit, SIGNAL( returnPressed()), this, SLOT( search()) );
	connect ( genreEdit, SIGNAL( returnPressed()), this, SLOT( search()) );
	connect ( manuEdit, SIGNAL( returnPressed()), this, SLOT( search()) );
	connect ( keywordEdit, SIGNAL( returnPressed()), this, SLOT( search()) );
}

SearchWidget::~SearchWidget()
{
	// stopping thread
	// cancel query first
	queryThread->cancelQuery();
	// stop eventloop
	queryThread->quit();
	// wait 2000 seconds to stop the eventloop
	queryThread->wait(2000);
	delete queryThread;
}

void SearchWidget::init ( QList< MediaPlugin* > newMediaPlugins, QList< SearchPlugin* > newSearchPlugins, ShoppingCart* shoppingCart )
{
	this->shoppingCart = shoppingCart;
	
  typeEdit->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
  genreEdit->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
  manuEdit->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
  userEdit->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
  mediaPlugins = newMediaPlugins;
  searchPlugins = newSearchPlugins;

  // add us as a left dock widget and create a entry to show and hide the dock widget
  QDockWidget *dockWidget = new QDockWidget ( tr ( "Search" ) );
	dockWidget->setObjectName("searchDockWidget");
  dockWidget->setWidget ( this );
  QAction *searchAction = new QAction ( tr ( "Search" ), this );
  emit addAction ( searchAction, "viewMenu" );
  searchAction->setCheckable ( true );
  connect ( dockWidget, SIGNAL ( visibilityChanged ( bool ) ), searchAction, SLOT ( setChecked ( bool ) ) );
  connect ( searchAction, SIGNAL ( triggered ( bool ) ), dockWidget, SLOT ( setVisible ( bool ) ) );
  emit addNewDockWidget ( dockWidget, Qt::LeftDockWidgetArea );

	QSettings settings;
	settings.beginGroup("SearchWidgetDefault");
	QString defaultType = settings.value ( "materialType" ).toString();
	QString defaultOrder = settings.value("order").toString();
	QString focused = settings.value("focused").toString();
	settings.endGroup();
  // add stuff from database
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
  if ( db.isOpen() ) {
    // add material types
    QSqlQuery query ( "SELECT id, name FROM materialtypes" );
    query.exec();
    while ( query.next() ) {
      typeMap.insert ( query.value ( 0 ).toInt(), query.value ( 1 ).toString() );
      typeEdit->addItem ( query.value ( 1 ).toString() );
    }
    
    // add genres
		genreModel = new QSqlQueryModel(this);
		completerModels << genreModel;
    genreModel->setQuery ( "SELECT id, name FROM genres ORDER BY name" );
    QCompleter *completer = new QCompleter(genreModel, this);
		completer->setCaseSensitivity ( Qt::CaseInsensitive );
		completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
		completer->setCompletionColumn(genreModel->record().indexOf("name"));
		genreEdit->setCompleter ( completer );

    // add manufacturer
		manuModel = new QSqlQueryModel(this);
		completerModels << manuModel;
    manuModel->setQuery ( "SELECT id, name FROM manufacturers ORDER BY name" );
    completer = new QCompleter(manuModel, this);
		completer->setCaseSensitivity ( Qt::CaseInsensitive );
		completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
		completer->setCompletionColumn(manuModel->record().indexOf("name"));
		manuEdit->setCompleter ( completer );

    // add users
    query.exec ( "SELECT id, name, givenName, userName FROM users" );
    while ( query.next() ) {
			if(query.record().value("userName").toString() == db.userName()){
				ownUserId = query.record().value("id").toUInt();
			}
			else{
				QString name = QString ( "%1, %2" )
						.arg ( query.record().value ( "name" ).toString() )
						.arg ( query.record().value ( "givenName" ).toString() );
				userMap.insert ( query.record().value ( "id" ).toInt(), name );
				userEdit->addItem ( name );
			}
    }
    if(!defaultOrder.isEmpty())
			userEdit->setCurrentIndex(userEdit->findText(defaultOrder));

		if(closeDB)
			db.close();
  }
	
	connect ( typeEdit, SIGNAL ( currentIndexChanged ( QString ) ), this, SLOT ( typeChanged ( QString ) ) );
	typeEdit->setCurrentIndex ( typeEdit->findText ( defaultType ) );
	if(typeEdit->currentText().isEmpty())
		typeEdit->setCurrentIndex(0);
	
	QList<QWidget*> widgets;
	widgets.append(titleEdit);
	widgets.append(personEdit);
	widgets.append(eanEdit);
	foreach(QWidget *widget, widgets){
		if(widget->objectName() == focused)
			widget->setFocus();
	}
}

void SearchWidget::typeChanged ( QString text )
{
	// remove old details widget
	if ( stackedWidget->currentWidget() ){
		disconnect(stackedWidget->currentWidget());
		stackedWidget->removeWidget ( stackedWidget->currentWidget() );
	}
	
	// add new one
  foreach ( MediaPlugin *plugin, mediaPlugins ) {
    if ( plugin->supportedMaterialTypes().contains ( text ) ) {
      SearchDetailsWidget *widget = plugin->searchWidget( text );
      if ( widget ){
				widget->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
        stackedWidget->addWidget ( widget );
				connect(widget, SIGNAL(startSearch()), this, SLOT(search()));
			}
      return;
    }
  }
}

void SearchWidget::search()
{
  QByteArray geometry = saveGeometry();
	if(ownUserId == 0)
		return;
	if(!genreModel)
		return;
	
	// clear out old resultsummary
// 	emit setCentralWidget(0);
	
	QList<QWidget*> filledWidgets;
  QList<SqlConstraint> constraints;
	SqlConstraint constraint;
  if ( !titleEdit->text().isEmpty() ){
    constraint.setConstraint("title LIKE ?");
		constraint.addBindValue(QString("%%1%").arg ( titleEdit->text() ) );
		constraints << constraint;
		filledWidgets.append(titleEdit);
	}
	if(	!personEdit->text().isEmpty() ){
		constraint = SqlConstraint();
		constraint.setConstraint("(name LIKE ? OR givenName LIKE ?)" );
		constraint.addBindValue(QString("%%1%").arg( personEdit->text() ));
		constraint.addBindValue(QString("%%1%").arg( personEdit->text() ));
		constraint.setNeedPersons();
		constraints << constraint;
		filledWidgets.append(personEdit);
	}
  if ( !eanEdit->text().isEmpty() ){
		if(!minValidator)
			minValidator = new MINValidator(this);

		// convert min
		QStringList mins;
		mins << minValidator->strip(eanEdit->text());
		foreach(QString possibleType, minValidator->possibleTypes(mins[0])){
			QString converted = minValidator->convert(mins[0], possibleType);
			if(!converted.isEmpty())
				mins << converted;
		}

		// prepare all mins
		QStringList conditions;
		foreach(QString min, mins)
			conditions << "min LIKE ?";
		
		constraint = SqlConstraint();
    constraint.setConstraint(QString("id IN(SELECT DISTINCT materialsId FROM materialhasmins WHERE %1 ORDER BY materialsId)")
															.arg(conditions.join(" OR ")) );
		foreach(QString min, mins)
			constraint.addBindValue(QString("%%1%").arg ( min ) );
		constraints << constraint;
		filledWidgets.append(eanEdit);
	}
  if( !genreEdit->text().isEmpty() ){
		unsigned int id = 0;
		for(int i = 0; i < genreModel->rowCount(); i++){
			if(genreModel->record(i).value("name").toString() == genreEdit->text()){
				id = genreModel->record(i).value("id").toUInt();
				break;
			}
		}
		if(id == 0){
			genreEdit->setText("");
		}
		else{
			constraint = SqlConstraint();
			constraint.setConstraint("genresId=?");
			constraint.addBindValue(id);
			constraints << constraint;
		}
	}
  if( !manuEdit->text().isEmpty() ){
		unsigned int id = 0;
		for(int i = 0; i < manuModel->rowCount(); i++){
			if(manuModel->record(i).value("name").toString() == manuEdit->text()){
				id = manuModel->record(i).value("id").toUInt();
				break;
			}
		}
		if(id == 0){
			manuEdit->setText("");
		}
		else{
			constraint = SqlConstraint();
			constraint.setConstraint("manufacturersId=?");
			constraint.addBindValue(id);
			constraints << constraint;
		}
	}
	if( !keywordEdit->text().isEmpty() ){
		QStringList keywordList = keywordEdit->text().split(" ");
		QStringList constraintList;
		foreach(QString string, keywordList)
			constraintList <<  "id IN(SELECT materialsId FROM viewmaterialskeywords WHERE keyword=?)";
		constraint = SqlConstraint();
		constraint.setConstraint( constraintList.join(" AND ") );
		foreach(QString string, keywordList)
			constraint.addBindValue(string);
		constraints << constraint;
	}
  int userId = 0;
	switch(userEdit->currentIndex()){
		case 0:
			userId = 0;
			break;
		case 1:
			userId = -2;
			break;
		case 2:
			userId = -1;
			break;
		case 3:
			userId = ownUserId;
			break;
		default:
			userId = userMap.key(userEdit->currentText());
			break;
	}
	switch(statusEdit->currentIndex()){
		case 1:
			constraint = SqlConstraint();
			constraint.setConstraint("wish=?" );
			constraint.addBindValue(1);
			constraints << constraint;
			break;
		case 2:
			constraint = SqlConstraint();
			if(userId <= 0){
				constraint.setConstraint( "(wish=? OR (presenteeId>0 AND presenteeId<>?))" );
				constraint.addBindValue(1);
				constraint.addBindValue(ownUserId);
			}
			else{
				constraint.setConstraint( "((usersId=? AND wish=?) OR (presenteeId=? AND presenteeId<>?))" );
				constraint.addBindValue(userId);
				constraint.addBindValue(1);
				constraint.addBindValue(userId);
				constraint.addBindValue(ownUserId);
			}
			constraints << constraint;
			break;
		case 3:
			constraint = SqlConstraint();
			constraint.setConstraint("literatur" );
			constraint.addBindValue(1);
			constraints << constraint;
			break;
		case 4:
			constraint = SqlConstraint();
			constraint.setConstraint("(wish=? OR giftOnly=? OR literatureOnly=?)" );
			constraint.addBindValue(1);
			constraint.addBindValue(1);
			constraint.addBindValue(1);
			constraints << constraint;
			break;
		case 5:
			constraint = SqlConstraint();
			constraint.setConstraint("(wish<>? AND giftOnly<>? AND literatureOnly<>?)" );
			constraint.addBindValue(1);
			constraint.addBindValue(1);
			constraint.addBindValue(1);
			constraints << constraint;
			break;
	}
  
  MediaModel *model = 0;
  if ( typeEdit->currentIndex() == 0 ) {
    // search all
    model = new MediaModel(queryThread, this);
  }
  else {
    MediaPlugin *mediaPlugin = 0;
    foreach ( MediaPlugin *plugin, mediaPlugins ) {
      if ( plugin->supportedMaterialTypes().contains ( typeEdit->currentText() ) ) {
        mediaPlugin = plugin;
        break;
      }
    }
    if ( !mediaPlugin ) {
      emit error ( tr ( "Could not find a plugin which handles the selected media type" ) );
      return;
    }

    model = mediaPlugin->newModel( typeEdit->currentText(), queryThread );
  }
  if(!model){
    emit error( tr ( "No model created!" ) );
    return;
  }
  connectModel(model);
	
	// show result view
	if(!resSum){
		resSum = new ResultSummary ( mediaPlugins, this );
		connectByPass ( resSum );
		resSum->init(shoppingCart);
		emit setCentralWidget ( resSum );
	}
	resSum->setResultset(model);
	
	// start search
	SearchDetailsWidget *widget = 0;
	if(detailsBox->isChecked())
		widget = qobject_cast<SearchDetailsWidget*>(stackedWidget->widget(0));
  model->load(constraints, searchPlugins, userId, widget);
	
	// store current stati
	// check which widget has the focuse
	QString focused;
	if(filledWidgets.size() == 1){
		focused = filledWidgets[0]->objectName();
	}
	else{
		if(titleEdit->hasFocus())
			focused = titleEdit->objectName();
		if(personEdit->hasFocus())
			focused = personEdit->objectName();
		if(eanEdit->hasFocus())
			focused = eanEdit->objectName();
	}
	
	restoreGeometry(geometry);
	
	QSettings settings;
	settings.beginGroup("SearchWidgetDefault");
	settings.setValue("materialType", typeEdit->currentText());
	settings.setValue("order", userEdit->currentText());
	settings.setValue("focused", focused);
	settings.endGroup();
}

void SearchWidget::reloadForeignKeys()
{
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return;
	foreach(QSqlQueryModel *queryModel, completerModels)
		queryModel->setQuery(queryModel->query().lastQuery());
	if(closeDB)
		db.close();
}

void SearchWidget::setupQueryThread()
{
	MSqlDatabase db = MSqlDatabase::database();
	queryThread->dbSetup(db.databaseName(), db.userName(), db.password(), db.hostName(), db.driverName());
}

void SearchWidget::connectModel(QObject* model)
{
  connect(model, SIGNAL(error(QString)), this, SLOT(errorByPass(QString)));
  connect(model, SIGNAL(warning(QString)), this, SLOT(warningByPass(QString)));
  connect(model, SIGNAL(information(QString)), this, SLOT(informationByPass(QString)));
	
	connect(model, SIGNAL(showStatusWidget(QWidget*)), this, SLOT(showStatusWidgetByPass(QWidget*)));
	connect(model, SIGNAL(removeStatusWidget(QWidget*)), this, SLOT(removeStatusWidgetByPass(QWidget*)));
	connect(model, SIGNAL(dataChanged(QModelIndex, QModelIndex)), this, SLOT(reloadForeignKeys()));
}

#include "searchwidget.moc"
