/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


 #include "config.h"
 
#include "settingsdialog.h"
#include "settingsgeneral.h"
#include <mediaplugin.h>
#include <searchplugin.h>

#include <QFileDialog>
#include <QPluginLoader>
#include <QSettings>

SettingsDialog::SettingsDialog (QList< MediaPlugin* > mediaPlugins, QList< SearchPlugin* > searchPlugins, QWidget* parent, Qt::WindowFlags fl )
	: MDialog ( parent, fl )
{
	this->mediaPlugins = mediaPlugins;
	this->searchPlugins = searchPlugins;
	
  setupUi ( this );
  connect(buttonCancel, SIGNAL(clicked()), this, SLOT(cancel()));
  connect(buttonSave, SIGNAL(clicked()), this, SLOT(save()));
  connect(buttonSaveAndQuit, SIGNAL(clicked()), this, SLOT(saveQuit()));
  
  connect(sectionList, SIGNAL(currentRowChanged(int)), this, SLOT(changeContent(int)));
  addContent();
  sectionList->setCurrentRow(0);
	
	QSettings settings;
	settings.beginGroup("geometries");
	restoreGeometry(settings.value(this->objectName()).toByteArray());
	settings.endGroup();
	
	setHelp("keyword:Settings", helpButton);
}

SettingsDialog::~SettingsDialog(){
	close();
  qDeleteAll(widgets);
}

void SettingsDialog::closeEvent(QCloseEvent* event)
{
	QSettings settings;
	settings.beginGroup("geometries");
	settings.setValue(this->objectName(), saveGeometry());
	settings.endGroup();
	QDialog::closeEvent(event);
}

bool SettingsDialog::saveSettings(){
  setError("");
  bool result = true;
  for(int i = 0; i < widgets.size(); i++){
    if(!widgets[i]->saveSettings()){
      result = false;
    }
  }
  if(result)
    emit reloadSettings();
  return result;
}

void SettingsDialog::saveQuit(){
  if(saveSettings())
    close();
}

void SettingsDialog::cancel(){
  close();
}

void SettingsDialog::addContent(){
  // remove the empty widget, which was created by the qt designer
  QWidget *remove = stackLayout->widget(0);
	if(remove){
		stackLayout->removeWidget(remove);
		delete remove;
	}

  // add the general widget as first stack widget
  widgets.append(new SettingsGeneral(this));
  stackLayout->addWidget(widgets[0]);
  sectionList->addItem(widgets[0]->name());

  // then add the plugin-tabs to the correspondend tabwidgets
  addPlugins();

  // resize sectionList
  int maxSize = 0;
  QFontMetrics fontMetric = sectionList->fontMetrics();
  for(int i = 0; i < sectionList->count(); i++){
    if(maxSize < fontMetric.width(sectionList->item(i)->text()))
      maxSize = fontMetric.width(sectionList->item(i)->text());
  }
  maxSize += sectionList->iconSize().width();
  sectionList->setMinimumWidth(maxSize);
  sectionList->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);
}

void SettingsDialog::changeContent(int row){
  if(row > -1){
    stackLayout->setCurrentIndex(row);
  }
}

void SettingsDialog::addPlugins(){
  foreach (MediaPlugin* module, mediaPlugins){
    SettingsWidget *tmp = module->settingsWidget();
    if(tmp){
			tmp->setParent(this);
      widgets.append(tmp);
      stackLayout->addWidget(tmp);
      sectionList->addItem(new QListWidgetItem(module->icon(), module->name()));
      connect(tmp, SIGNAL(setError(QString)), this, SLOT(setError(QString)));
      connectByPass(tmp);
    }
  }
  
	foreach (SearchPlugin* module, searchPlugins){
    SettingsWidget *tmp = module->settingsWidget();
    if(tmp){
      widgets.append(tmp);
      stackLayout->addWidget(tmp);
      sectionList->addItem(new QListWidgetItem(module->icon(), module->name()));
      connect(tmp, SIGNAL(setError(QString)), this, SLOT(setError(QString)));
      connectByPass(tmp);
    }
  }
}

void SettingsDialog::setError(QString msg){
  errorLabel->setText(msg);
}

#include "settingsdialog.moc"
