/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "meganizerconfig.h"

#include "settingsgeneral.h"
#include "ui_settingsgeneral.h"

#include "setupuserwidget.h"
#include <dbsettings.h>
#include <mimetypemodel.h>
#include <searchexternalapp.h>
#include <openfileexternal.h>

#include <QFileDialog>
#include <QSqlDatabase>
#include <QDir>
#include <QFileInfo>
#include <QLocale>

SettingsGeneral::SettingsGeneral ( QWidget *parent, Qt::WindowFlags flags ) : SettingsWidget ( parent, flags ),
    m_ui ( new Ui::SettingsGeneral )
{

  init();

  m_ui->cancelButton->setVisible ( false );
  m_ui->saveButton->setVisible ( false );
  m_ui->saveAndQuitButton->setVisible ( false );
}

SettingsGeneral::SettingsGeneral ( bool showButtons, QWidget *parent, Qt::WindowFlags flags ) : SettingsWidget ( parent, flags ),
    m_ui ( new Ui::SettingsGeneral )
{

  init();
  connect ( m_ui->cancelButton, SIGNAL ( clicked() ), this, SLOT ( close() ) );
  connect ( m_ui->saveButton, SIGNAL ( clicked() ), this, SLOT ( saveSettings() ) );
  connect ( m_ui->saveAndQuitButton, SIGNAL ( clicked() ), this, SLOT ( saveQuit() ) );
}

SettingsGeneral::~SettingsGeneral()
{
  delete m_ui;
}

void SettingsGeneral::init()
{
  setName ( tr ( "General" ) );
  m_ui->setupUi ( this );

  connect ( m_ui->pluginPathSearch, SIGNAL ( clicked() ), this, SLOT ( searchPluginDir() ) );
#if QT_VERSION_MINOR < 6
	m_ui->pluginPathSearch->setIcon(QIcon(":/icons/find.png"));
#else
	m_ui->pluginPathSearch->setIcon(QIcon::fromTheme("edit-find",
																				 QIcon(":/icons/find.png")));
#endif
if(!iconsAndText){
	if(!m_ui->pluginPathSearch->icon().isNull())
		m_ui->pluginPathSearch->setText("");
}

  // add database stuff
  dbSettings = new DBSettings ( 0, m_ui->tabView );
  connectByPass ( dbSettings );
  m_ui->tabView->addTab ( dbSettings, dbSettings->name() );

  userWidget = new SetupUserWidget ( 0, m_ui->tabView );
  connectByPass ( userWidget );
  m_ui->tabView->addTab ( userWidget, userWidget->name() );

	// getting all available documentation languages
	// (it should be the same as all other available languages)
	QDir docDir(QString(SHAREDIR) + "/lang");
	if ( docDir.exists() ) {
    QStringList filter;
    filter.append ( "meganizer_*.qm" );
    QStringList newFiles = docDir.entryList ( filter, QDir::Files | QDir::Readable );
    foreach ( QString file, newFiles ) {
      // Needed for the translation of the app
      QRegExp reg("_([a-zA-Z]{2})\\.qm");
			if(reg.indexIn(file) > 0)
				m_ui->defaultLangEdit->addItem(reg.cap(1).toUpper());
		}
		// if no english version is present, add it,
		// because english is builtin
		if(m_ui->defaultLangEdit->findText("EN") < 0)
			m_ui->defaultLangEdit->addItem("EN");
  }
	
  loadSettings();
}

void SettingsGeneral::loadSettings()
{
  m_ui->pluginPathEdit->setText ( qSettings.value ( "pluginDir" ).toString() );
  m_ui->iconsAndTextCheckBox->setChecked ( qSettings.value ( "iconsAndText" ).toBool() );
  m_ui->alternatingRowColors->setChecked ( qSettings.value ( "alternatingRowColors" ).toBool() );
	QLocale qLocale;
	m_ui->defaultLangEdit->setCurrentIndex(
		m_ui->defaultLangEdit->findText(
			qSettings.value (
				"defaultLang",
				qLocale.name().left(2).toUpper() 
			).toString().toUpper()
		)
	);
#if QT_VERSION_MINOR < 6
	m_ui->usedBorderEdit->setValue( qSettings.value( "usedBorder", 30 ).toDouble() );
	m_ui->minUsedBorderEdit->setValue( qSettings.value( "minUsedBorder", 80 ).toDouble() );
#else
	m_ui->usedBorderEdit->setValue( qSettings.value( "usedBorder", 30 ).toFloat() );
	m_ui->minUsedBorderEdit->setValue( qSettings.value( "minUsedBorder", 80 ).toFloat() );
#endif
  
  // mimetype stuff
  qSettings.beginGroup("OpenFileExternal");
  m_ui->showAnytimeEdit->setChecked(qSettings.value("showAnytime", false).toBool());
  // load path list
  QList<QDir> dirs = SearchExternalApp::defaultPaths();
  QStringList paths;
  foreach(QDir dir, dirs){
    paths.append(dir.absolutePath());
  }
  m_ui->searchPathsEdit->addItems(paths);
  // load known mimetypes
  MimeTypeModel *mimeModel = OpenFileExternal::mimeTypes();
  m_ui->mimeTypesEdit->setModel(mimeModel);
  qSettings.endGroup();
}

void SettingsGeneral::changeEvent ( QEvent *e )
{
  QWidget::changeEvent ( e );

  switch ( e->type() ) {
    case QEvent::LanguageChange:
      m_ui->retranslateUi ( this );
      break;
    default:
      break;
  }
}

void SettingsGeneral::searchPluginDir()
{
  QString dir = QFileDialog::getExistingDirectory ( this, tr ( "Select directory with plugins" ), m_ui->pluginPathEdit->text() );

  if ( !dir.isEmpty() ) {
    m_ui->pluginPathEdit->setText ( dir );
  }
}

bool SettingsGeneral::saveSettings()
{
  qSettings.setValue ( "pluginDir", m_ui->pluginPathEdit->text() );
  qSettings.setValue ( "iconsAndText", m_ui->iconsAndTextCheckBox->isChecked() );
  qSettings.setValue ( "alternatingRowColors", m_ui->alternatingRowColors->isChecked() );
	qSettings.setValue ( "defaultLang", m_ui->defaultLangEdit->currentText().toLower() );
  qSettings.setValue ( "usedBorder", m_ui->usedBorderEdit->value() );
	qSettings.setValue ( "minUsedBorder", m_ui->minUsedBorderEdit->value() );

  // mimetype stuff
  qSettings.beginGroup("OpenFileExternal");
  qSettings.setValue("showAnytime", m_ui->showAnytimeEdit->isChecked());
  qSettings.endGroup();
  
  bool result = true;

  if ( !dbSettings->saveSettings() )
    result = false;

  if ( !userWidget->saveSettings() )
    result = false;

  return result;
}

void SettingsGeneral::saveQuit()
{
  if ( saveSettings() )
    close();
}

void SettingsGeneral::closeEvent ( QCloseEvent *event )
{
  emit ready();
  event->accept();
}

#include "settingsgeneral.moc"
