/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "eanvalidator.h"

#include <QObject>
#include <QtPlugin>
#include <QRegExp>

EANValidator::EANValidator()
{

}

QString EANValidator::type() const
{
	return "EAN";
}

QString EANValidator::description() const
{
	return tr("EAN + ISBN-13 - European Product Number is used for many kind of products. It can have 8 or 13 numbers. The ISBN-13 is also an EAN");
}

bool EANValidator::canConvert(QString toType) const
{
	return false;
}

QString EANValidator::convert(QString min, QString toType) const
{
	return QString();
}

bool EANValidator::isValid(QString min) const
{
	min.replace(QRegExp("[\\-\\s]"), "");
	
	if((min.length() != 8) && (min.length() != 13))
		return false;
	
	if(checksum(min) == min.right(1).toInt())
		return true;
	
	return false;
}

QString EANValidator::validate(QString min) const
{
	min = strip(min);
	
	if(isValid(min))
		return min;
	
	if(min.length() < 7)
		return QString();
	
	if((min.length() > 8) && (min.length() < 12))
		return QString();
	
	if(min.length() > 13)
		return QString();
	
	if((min.length() == 8) || (min.length() == 13))
		min = min.left(min.length() - 1);
	
	min += QString::number(checksum(min));
	if(isValid(min))
		return min;
	return QString();
}

bool EANValidator::isType(QString min) const
{
	if(isValid(min))
		return true;
	return false;
}

bool EANValidator::mayType(QString min) const
{
	min = strip(min);
	
	if(isValid(min))
		return true;
	
	if(min.length() < 13)
		return true;
	
	if(isValid(validate(min)))
		return true;
	
	return false;
}

QString EANValidator::strip(QString min) const
{
	min = min.trimmed();
	min.replace(QRegExp("[^\\d]"), "");
	return min;
}

bool EANValidator::operator==(const MINValidatorPlugin& plugin) const
{
	if(type() == plugin.type())
		return true;
	return false;
}

bool EANValidator::operator!=(const MINValidatorPlugin& plugin) const
{
	return !operator==(plugin);
}

int EANValidator::checksum(QString min) const
{
	min = strip(min);
	
	// remove last digest, because checksum isn't needed
	if((min.length() == 8) || (min.length() == 13))
		min = min.left(min.length() - 1);
		
	if((min.length() != 7) && (min.length() != 12))
		return -1;
	
	int sumEven = 0;
	int sumOdd = 0;
	for(int i = 0; i < min.length(); i++){
		// because of i starts at 0, even numbers are not devideable by 2
		if(i % 2 != 0){
			sumEven += min.mid(i, 1).toInt();
		}
		else{
			sumOdd += min.mid(i, 1).toInt();
		}
	}
	int sum = sumOdd + 3 * sumEven;
	if(sum < 1)
		return -1;
	int result = 0;
	if(sum % 10 != 0)
		result = 10 - (sum % 10);

	return result;
}

#include "eanvalidator.moc"

Q_EXPORT_PLUGIN2 ( Meganizer_EAN, EANValidator )
