/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "amazonconntypesettings.h"

#include "configamazon.h"
#include <mediaplugin.h>
#include <mediamodel.h>
#include "amazon.h"
#include "connectmaterialtype.h"

#include <QDir>
#include <QPluginLoader>

AmazonConnTypeSettings::AmazonConnTypeSettings(Amazon* amazon, QWidget* parent, Qt::WindowFlags flags)
	: SettingsWidget(parent, flags)
{
	setName(tr("Type Connections"));
	
	this->amazon = amazon;
	if(!amazon)
		return;
	
	setupUi(this);
	connMatType = qobject_cast<ConnectMaterialType*>(connectMaterialType);
	connectByPass(connMatType);
	
	// create list with plugins
	QSettings settings;
  QString pluginDir = settings.value ( "pluginDir" ).toString();

  if ( pluginDir.isEmpty() ) {
    pluginDir = QString ( PLUGINDIR );
  }
  
  //load and test plugin
  QDir dir ( pluginDir );

  if ( !dir.exists() ) {
    emit error ( tr ( "Plugin dir does not exist!\n%1" ).arg ( pluginDir ) );
    return;
  }

  QStringList files = dir.entryList ( QDir::Files );

  if ( files.size() < 1 ) {
    emit error ( tr ( "No files found at \n%1" ).arg ( pluginDir ) );
    return;
  }

	QStringList types;
  foreach ( QString file, files ) {
		QPluginLoader loader ( dir.filePath(file) );
		QObject *plugin = loader.instance();
    if ( !plugin ){
			continue;
		}

		// first try to find media plugins
		MediaPlugin *newPlugin = qobject_cast<MediaPlugin *> ( plugin );

		if ( !newPlugin ){
			continue;
		}
		
		mediaPlugins.append(newPlugin);
		types << newPlugin->supportedMaterialTypes();
  }
  
  types.sort();
	typeEdit->addItems(types);
	
	connect(typeEdit, SIGNAL(currentIndexChanged(int)), this, SLOT(showConnType()));
	showConnType();
}
  
AmazonConnTypeSettings::~AmazonConnTypeSettings()
{

}


bool AmazonConnTypeSettings::saveSettings()
{
	return connMatType->saveSettings();
}

void AmazonConnTypeSettings::showConnType()
{
	if(!amazon)
		return;
	
	connMatType->saveSettings();
	
	QString type = typeEdit->currentText();
	foreach(MediaPlugin *plugin, mediaPlugins){
		if(!plugin->supportedMaterialTypes().contains(type))
			continue;
		
		MediaModel *model = plugin->newModel(type);
		Medium *med = model->newMedium();
		connMatType->setPlugins(med->group(), amazon);
		delete med;
		delete model;
	}
}

#include "amazonconntypesettings.moc"
