/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DEMOSHOP_H
#define DEMOSHOP_H


#include "searchplugin.h"

class MediaModel;
class SettingsWidget;
class SetupWizardObject;
class Medium;

#include <QString>
#include <QStringList>
#include <QMultiMap>
#include <QtPlugin>
#include <QObject>

/**
	* Used to test features and show possible new features
  */
class DemoShop : public SearchPlugin
{
	Q_OBJECT
	Q_INTERFACES ( SearchPlugin )

  public:
    DemoShop ( QObject *parent = 0 );
    ~DemoShop();

    bool initiate();
    bool unload();
    QString name() const;
    static QString group();
    QString settingsGroup();
    QIcon icon();
    QString description();
    QMap<QString, QString> supportedAttributes();
    QMap<QString, QString> supportedCategories();
    SettingsWidget *settingsWidget();
    SetupWizardObject *wizardObject();
    bool search ( MediaModel *resultModel, QList<SqlConstraint> sqlConstraints );
    void recieveBuyData ( Medium *medium );
    bool purchase ( QList<BuyData> items, QMap<Medium*, int> quantities );
    static QMultiMap<QString, QString> locales();
    static QMultiMap<QString, QString> localesMerchantId();
    bool operator== (const SearchPlugin &plugin) const;
    bool operator!= (const SearchPlugin &plugin) const;

  public slots:
    void show();

    // MObject slots
    void settingsChanged();
    void print();
    void save();
    void doExport();
    void redo();
    void undo();
    void cut();
    void copy();
    void paste();
    void reload();
    void reloadSettingsByPass();

  signals:
    void foundBuyData ( const BuyData &data );

    void error ( const QString &string );
    void warning ( const QString &string );
    void information ( const QString &string );

    void addMenu ( QMenu* newMenu, const QString &parent );
    void delMenu ( QMenu *del );
    void addAction ( QAction* newMenu, const QString &parent );
    void delAction ( QAction *del );
    void addToolbar ( QToolBar* newbar );
    void delToolbar ( QToolBar* delBar );

    void addNewDockWidget ( QDockWidget *widget, const Qt::DockWidgetArea &area );
    void setCentralWidget ( QWidget *centralWidget );
    void removeDockWidget ( QDockWidget *widget );

    void dbLogin();

    void showSettings();
    void reloadSettings();

    void showHelp( const QString &page );

    void showStatusWidget ( QWidget *widget );
    void removeStatusWidget ( QWidget *widget );
};
#endif // DEMOSHOP_H
